/*
 * n01_stats
 *
 * data.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <tchar.h>
#include <stdio.h>

#include "general.h"
#include "String.h"
#include "Memory.h"
#include "Message.h"
#include "file.h"
#include "csv.h"
#include "data.h"
#include "output.h"

#include "resource.h"

/* Define */
#define DATA_CSV_FILE				TEXT("n01_data.csv")
#define LIST_CSV_FILE				TEXT("n01_list.csv")

/* Global Variables */
extern HINSTANCE hInst;
extern OPTION op;
extern TCHAR csv_path[];
extern TCHAR out_data_path[];
extern TCHAR out_list_path[];

extern DATA_INFO *g_di;
extern int g_di_count;
extern DATA_INFO *g_show_di;
extern int g_show_di_count;

extern EXTRACT_INFO exi;

/* Local Function Prototypes */
static BOOL data_split_set(DATA_INFO *di, const int *split_index, const int cnt, const int index);

/*
 * data_get_title - ^Cg擾
 */
TCHAR *data_get_title(const int type)
{
	static TCHAR ret[BUF_SIZE];

	switch (type) {
	case HDTP_NAME:
		message_copy_res(IDS_STRING_HDTP_NAME, ret);
		break;
	case HDTP_LINK_NAME:
		message_copy_res(IDS_STRING_HDTP_LINK_NAME, ret);
		break;
	case HDTP_VS_NAME:
		message_copy_res(IDS_STRING_HDTP_VS_NAME, ret);
		break;
	case HDTP_DATE:
		message_copy_res(IDS_STRING_HDTP_DATE, ret);
		break;
	case HDTP_FILE_NAME:
		message_copy_res(IDS_STRING_HDTP_FILE_NAME, ret);
		break;
	case HDTP_START_SCORE:
		message_copy_res(IDS_STRING_HDTP_START_SCORE, ret);
		break;
	case HDTP_SETS_PLAYED:
		message_copy_res(IDS_STRING_HDTP_SETS_PLAYED, ret);
		break;
	case HDTP_SETS_WON:
		message_copy_res(IDS_STRING_HDTP_SETS_WON, ret);
		break;
	case HDTP_LEGS_PLAYED:
		message_copy_res(IDS_STRING_HDTP_LEGS_PLAYED, ret);
		break;
	case HDTP_LEGS_WON:
		message_copy_res(IDS_STRING_HDTP_LEGS_WON, ret);
		break;
	case HDTP_TON:
		message_copy_res(IDS_STRING_HDTP_TON, ret);
		break;
	case HDTP_TON00:
		message_copy_res(IDS_STRING_HDTP_TON00, ret);
		break;
	case HDTP_TON40:
		message_copy_res(IDS_STRING_HDTP_TON40, ret);
		break;
	case HDTP_TON80:
		message_copy_res(IDS_STRING_HDTP_TON80, ret);
		break;
	case HDTP_HIGH_OFF:
		message_copy_res(IDS_STRING_HDTP_HIGH_OFF, ret);
		break;
	case HDTP_SHORT_GAME:
		message_copy_res(IDS_STRING_HDTP_SHORT_GAME, ret);
		break;
	case HDTP_LONG_GAME:
		message_copy_res(IDS_STRING_HDTP_LONG_GAME, ret);
		break;
	case HDTP_AVG_SET:
		message_copy_res(IDS_STRING_HDTP_AVG_SET, ret);
		break;
	case HDTP_AVG_LEG:
		message_copy_res(IDS_STRING_HDTP_AVG_LEG, ret);
		break;
	case HDTP_AVG_DARTS:
		message_copy_res(IDS_STRING_HDTP_AVG_DARTS, ret);
		break;
	case HDTP_AVG_SCORE:
		message_copy_res(IDS_STRING_HDTP_AVG_SCORE, ret);
		break;
	case HDTP_AVG_FIRST9:
		message_copy_res(IDS_STRING_HDTP_AVG_FIRST9, ret);
		break;
	case HDTP_AVG_CHECK_OUT:
	case HDTP_AVG_CHECK_OUT_VIEW:
		message_copy_res(IDS_STRING_HDTP_AVG_CHECK_OUT, ret);
		break;
	case HDTP_AVG_CHECK_OUT_COUNT:
		message_copy_res(IDS_STRING_HDTP_AVG_CHECK_OUT_COUNT, ret);
		break;
	case HDTP_AVG_KEEP_LEGS:
	case HDTP_AVG_KEEP_LEGS_VIEW:
		message_copy_res(IDS_STRING_HDTP_AVG_KEEP_LEGS, ret);
		break;
	case HDTP_AVG_KEEP_LEGS_COUNT:
		message_copy_res(IDS_STRING_HDTP_AVG_KEEP_LEGS_COUNT, ret);
		break;
	case HDTP_AVG_BREAK_LEGS:
	case HDTP_AVG_BREAK_LEGS_VIEW:
		message_copy_res(IDS_STRING_HDTP_AVG_BREAK_LEGS, ret);
		break;
	case HDTP_AVG_BREAK_LEGS_COUNT:
		message_copy_res(IDS_STRING_HDTP_AVG_BREAK_LEGS_COUNT, ret);
		break;
	case HDTP_SUCCESS_2_80:
		message_copy_res(IDS_STRING_HDTP_SUCCESS_2_80, ret);
		break;
	case HDTP_FAILURE_2_80:
		message_copy_res(IDS_STRING_HDTP_FAILURE_2_80, ret);
		break;
	case HDTP_SUCCESS_81_130:
		message_copy_res(IDS_STRING_HDTP_SUCCESS_81_130, ret);
		break;
	case HDTP_FAILURE_81_130:
		message_copy_res(IDS_STRING_HDTP_FAILURE_81_130, ret);
		break;
	case HDTP_SUCCESS_131:
		message_copy_res(IDS_STRING_HDTP_SUCCESS_131, ret);
		break;
	case HDTP_FAILURE_131:
		message_copy_res(IDS_STRING_HDTP_FAILURE_131, ret);
		break;
	case HDTP_LEGS:
		message_copy_res(IDS_STRING_HDTP_LEGS, ret);
		break;
	default:
		*ret = TEXT('\0');
		break;
	}
	return ret;
}

/*
 * data_to_value - f[^𕶎ɕϊ
 */
TCHAR *data_to_value(const int type, const DATA_INFO *di, TCHAR *ret)
{
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	int i;

	switch (type) {
	case HDTP_NAME:
		lstrcpy(ret, di->name);
		break;
	case HDTP_LINK_NAME:
		if (di >= &g_di[0] && di < &g_di[g_di_count]) {
			i = di->index;
		} else {
			i = -1;
			if (di->index >= 0) {
				i = g_di[di->index].index;
			}
		}
		if (i < 0) {
			lstrcpy(ret, INDEFINITE_CHAR);
		} else {
			lstrcpy(ret, g_di[i].name);
		}
		break;
	case HDTP_VS_NAME:
		if (di->vs_index < 0) {
			lstrcpy(ret, INDEFINITE_CHAR);
		} else {
			lstrcpy(ret, g_di[di->vs_index].name);
		}
		break;
	case HDTP_DATE:
		if (di->start_time.wYear <= 0) {
			lstrcpy(ret, INDEFINITE_CHAR);
		}else if (di->start_time.wMonth <= 0) {
			wsprintf(ret, TEXT("%d"), di->start_time.wYear);
		}else if (di->start_time.wDay <= 0) {
			wsprintf(ret, TEXT("%d/%02d"), di->start_time.wYear, di->start_time.wMonth);
		}else if (di->start_time.wHour <= 0 && di->start_time.wMinute <= 0 && di->start_time.wSecond <= 0) {
			GetDateFormat(0, 0, &di->start_time, NULL, ret, BUF_SIZE - 1);
		} else {
			GetDateFormat(0, 0, &di->start_time, NULL, date, BUF_SIZE - 1);
			GetTimeFormat(0, 0, &di->start_time, NULL, time, BUF_SIZE - 1);
			lstrcpy(ret, date);
			lstrcat(ret, TEXT(" "));
			lstrcat(ret, time);
		}
		break;
	case HDTP_FILE_NAME:
		if (*di->file_name == TEXT('\0')) {
			lstrcpy(ret, INDEFINITE_CHAR);
		} else {
			lstrcpy(ret, di->file_name);
		}
		break;
	case HDTP_START_SCORE:
		if (di->start_score <= 0) {
			lstrcpy(ret, INDEFINITE_CHAR);
		} else {
			_itot(di->start_score, ret, 10);
		}
		break;
	case HDTP_SETS_PLAYED:
		_itot(di->sets_play, ret, 10);
		break;
	case HDTP_SETS_WON:
		_itot(di->sets, ret, 10);
		break;
	case HDTP_LEGS_PLAYED:
		_itot(di->legs_play, ret, 10);
		break;
	case HDTP_LEGS_WON:
		_itot(di->legs, ret, 10);
		break;
	case HDTP_TON:
		_itot(di->ton_count, ret, 10);
		break;
	case HDTP_TON00:
		_itot(di->ton00_count, ret, 10);
		break;
	case HDTP_TON40:
		_itot(di->ton40_count, ret, 10);
		break;
	case HDTP_TON80:
		_itot(di->ton80_count, ret, 10);
		break;
	case HDTP_HIGH_OFF:
		_itot(di->high_off, ret, 10);
		break;
	case HDTP_SHORT_GAME:
		_itot(di->short_game, ret, 10);
		break;
	case HDTP_LONG_GAME:
		_itot(di->long_game, ret, 10);
		break;
	case HDTP_AVG_SET:
		if (di->sets_play > 0) {
			_stprintf(ret, TEXT("%.2f%%"), ((float)(di->sets) / (float)(di->sets_play)) * 100);
		} else {
			lstrcpy(ret, TEXT("0.00%"));
		}
		break;
	case HDTP_AVG_LEG:
		if (di->legs_play > 0) {
			_stprintf(ret, TEXT("%.2f%%"), ((float)(di->legs) / (float)(di->legs_play)) * 100);
		} else {
			lstrcpy(ret, TEXT("0.00%"));
		}
		break;
	case HDTP_AVG_DARTS:
		if (di->win_count > 0) {
			_stprintf(ret, TEXT("%.3f"), (float)(di->win_darts) / (float)(di->win_count));
		} else {
			lstrcpy(ret, TEXT("0.000"));
		}
		break;
	case HDTP_AVG_SCORE:
		if (di->all_darts > 0) {
			_stprintf(ret, TEXT("%.3f"), (float)(di->all_score) / (float)(di->all_darts));
		} else {
			lstrcpy(ret, TEXT("0.000"));
		}
		break;
	case HDTP_AVG_FIRST9:
		if (di->first9_darts > 0) {
			_stprintf(ret, TEXT("%.3f"), (float)(di->first9_score) / (float)(di->first9_darts));
		} else {
			lstrcpy(ret, TEXT("0.000"));
		}
		break;
	case HDTP_AVG_CHECK_OUT:
		if (di->check_out_aim > 0) {
			_stprintf(ret, TEXT("%.2f%%"), ((float)(di->check_out) / (float)(di->check_out_aim)) * 100);
		} else {
			lstrcpy(ret, TEXT("0.00%"));
		}
		break;
	case HDTP_AVG_CHECK_OUT_COUNT:
		wsprintf(ret, TEXT("(%d / %d)"), di->check_out, di->check_out_aim);
		break;
	case HDTP_AVG_CHECK_OUT_VIEW:
		if (di->check_out_aim > 0) {
			_stprintf(ret, TEXT("%.2f%% (%d / %d)"),
				((float)(di->check_out) / (float)(di->check_out_aim)) * 100, di->check_out, di->check_out_aim);
		} else {
			lstrcpy(ret, TEXT("0.00% (0 / 0)"));
		}
		break;
	case HDTP_AVG_KEEP_LEGS:
		if (di->all_keep_legs > 0) {
			_stprintf(ret, TEXT("%.2f%%"), ((float)(di->win_keep_legs) / (float)(di->all_keep_legs)) * 100);
		} else {
			lstrcpy(ret, TEXT("0.00%"));
		}
		break;
	case HDTP_AVG_KEEP_LEGS_COUNT:
		wsprintf(ret, TEXT("(%d / %d)"), di->win_keep_legs, di->all_keep_legs);
		break;
	case HDTP_AVG_KEEP_LEGS_VIEW:
		if (di->all_keep_legs > 0) {
			_stprintf(ret, TEXT("%.2f%% (%d / %d)"),
				((float)(di->win_keep_legs) / (float)(di->all_keep_legs)) * 100, di->win_keep_legs, di->all_keep_legs);
		} else {
			lstrcpy(ret, TEXT("0.00% (0 / 0)"));
		}
		break;
	case HDTP_AVG_BREAK_LEGS:
		if (di->all_break_legs > 0) {
			_stprintf(ret, TEXT("%.2f%%"), ((float)(di->win_break_legs) / (float)(di->all_break_legs)) * 100);
		} else {
			lstrcpy(ret, TEXT("0.00%"));
		}
		break;
	case HDTP_AVG_BREAK_LEGS_COUNT:
		wsprintf(ret, TEXT("(%d / %d)"), di->win_break_legs, di->all_break_legs);
		break;
	case HDTP_AVG_BREAK_LEGS_VIEW:
		if (di->all_break_legs > 0) {
			_stprintf(ret, TEXT("%.2f%% (%d / %d)"),
				((float)(di->win_break_legs) / (float)(di->all_break_legs)) * 100, di->win_break_legs, di->all_break_legs);
		} else {
			lstrcpy(ret, TEXT("0.00% (0 / 0)"));
		}
		break;
	case HDTP_SUCCESS_2_80:
		_itot(di->success_2_80, ret, 10);
		break;
	case HDTP_FAILURE_2_80:
		_itot(di->failure_2_80, ret, 10);
		break;
	case HDTP_SUCCESS_81_130:
		_itot(di->success_81_130, ret, 10);
		break;
	case HDTP_FAILURE_81_130:
		_itot(di->failure_81_130, ret, 10);
		break;
	case HDTP_SUCCESS_131:
		_itot(di->success_131, ret, 10);
		break;
	case HDTP_FAILURE_131:
		_itot(di->failure_131, ret, 10);
		break;
	case HDTP_LEGS:
		if (di->vs_index < 0) {
			lstrcpy(ret, INDEFINITE_CHAR);
		} else {
			wsprintf(ret, TEXT("%d-%d"), di->legs, g_di[di->vs_index].legs);
		}
		break;
	default:
		*ret = TEXT('\0');
		break;
	}
	return ret;
}

/*
 * data_extract_check - \Ώۂ`FbN
 */
BOOL data_extract_check(const DATA_INFO *di)
{
	FILETIME ft, ft_f;
	TCHAR buf[BUF_SIZE];
	TCHAR *p, *r;

	if (di->status == DATA_STATUS_DELETE) {
		// 폜ς
		return FALSE;
	}
	// OŒo
	if (*exi.name != TEXT('\0') && (exi.name_hash != di->name_hash || lstrcmpi(exi.name, di->name) != 0)) {
		return FALSE;
	}
	// w
	if (exi.date.wYear != 0 && exi.date.wYear != di->start_time.wYear) {
		return FALSE;
	}
	if (exi.date.wMonth != 0 && exi.date.wMonth != di->start_time.wMonth) {
		return FALSE;
	}
	if (exi.date.wDay != 0 && exi.date.wDay != di->start_time.wDay) {
		return FALSE;
	}
	// tB^
	if (exi.filter_name_enable != 0 && *exi.filter_name != TEXT('\0')) {
		lstrcpy(buf, exi.filter_name);
		for (r = p = buf; *p != TEXT('\0'); p++) {
			if (*p == TEXT(',')) {
				*p = TEXT('\0');
				if (str_match(r, di->name) == TRUE) {
					break;
				}
				r = p + 1;
			}
		}
		if (*p == TEXT('\0')) {
			if (str_match(r, di->name) == FALSE) {
				return FALSE;
			}
		}
	}
	if (exi.filter_vs_name_enable != 0 && *exi.filter_vs_name != TEXT('\0') && di->vs_index >= 0) {
		lstrcpy(buf, exi.filter_vs_name);
		for (r = p = buf; *p != TEXT('\0'); p++) {
			if (*p == TEXT(',')) {
				*p = TEXT('\0');
				if (str_match(r, g_di[di->vs_index].name) == TRUE) {
					break;
				}
				r = p + 1;
			}
		}
		if (*p == TEXT('\0')) {
			if (str_match(r, g_di[di->vs_index].name) == FALSE) {
				return FALSE;
			}
		}
	}
	if (exi.filter_file_name_enable != 0 && *exi.filter_file_name != TEXT('\0')) {
		lstrcpy(buf, exi.filter_file_name);
		for (r = p = buf; *p != TEXT('\0'); p++) {
			if (*p == TEXT(',')) {
				*p = TEXT('\0');
				if (str_match(r, di->file_name) == TRUE) {
					break;
				}
				r = p + 1;
			}
		}
		if (*p == TEXT('\0')) {
			if (str_match(r, di->file_name) == FALSE) {
				return FALSE;
			}
		}
	}
	if (exi.filter_date_enable != 0) {
		SystemTimeToFileTime(&di->start_time, &ft);
		SystemTimeToFileTime(&exi.filter_date_st, &ft_f);
		if (CompareFileTime(&ft, &ft_f) < 0) {
			return FALSE;
		}
		SystemTimeToFileTime(&exi.filter_date_en, &ft_f);
		if (CompareFileTime(&ft, &ft_f) > 0) {
			return FALSE;
		}
	}
	if (exi.filter_start_score_enable != 0 && exi.filter_start_score != di->start_score) {
		return FALSE;
	}
	return TRUE;
}

/*
 * data_marge_check - }[WΏۂ`FbN
 */
BOOL data_marge_check(const DATA_INFO *di, const DATA_INFO *from_di)
{
	if (di->name_hash != from_di->name_hash || lstrcmpi(di->name, from_di->name) != 0) {
		return FALSE;
	}
	if (exi.sub_type == EXTRACT_SUB_TYPE_DAY) {
		if (di->start_time.wYear == from_di->start_time.wYear &&
			di->start_time.wMonth == from_di->start_time.wMonth &&
			di->start_time.wDay == from_di->start_time.wDay) {
			return TRUE;
		}
	} else if (exi.sub_type == EXTRACT_SUB_TYPE_MONTH) {
		if (di->start_time.wYear == from_di->start_time.wYear &&
			di->start_time.wMonth == from_di->start_time.wMonth) {
			return TRUE;
		}
	} else if (exi.sub_type == EXTRACT_SUB_TYPE_YEAR) {
		if (di->start_time.wYear == from_di->start_time.wYear) {
			return TRUE;
		}
	} else {
		return TRUE;
	}
	return FALSE;
}

/*
 * data_create_stats - \f[^𐶐
 */
BOOL data_create_stats(DATA_INFO *from_di, const int st, const int en, DATA_INFO *di, int *ret, const BOOL update)
{
	int i, j;

	for (i = st; i < en; i++) {
		// õ`FbN
		if (data_extract_check(&from_di[i]) == FALSE) {
			continue;
		}
		if (exi.sub_type == EXTRACT_SUB_TYPE_ALL) {
			// }[WȂŕ\
			if (update == FALSE) {
				// ǉ
				j = *ret;
				(*ret)++;
			} else {
				// XV
				for (j = 0; j < *ret; j++) {
					if (di[j].status == DATA_STATUS_DELETE) {
						// 폜ς
						continue;
					}
					if (di[j].index == i) {
						break;
					}
				}
				if (j >= *ret) {
					// ǉ
					j = *ret;
					(*ret)++;
				}
			}
			di[j] = from_di[i];
			di[j].index = i;
			continue;
		}
		// }[WΏۂ̌
		for (j = 0; j < *ret; j++) {
			if (di[j].status == DATA_STATUS_DELETE) {
				// 폜ς
				continue;
			}
			if (data_marge_check(&di[j], &from_di[i]) == TRUE) {
				break;
			}
		}
		if (j >= *ret) {
			// VKǉ
			j = *ret;
			(*ret)++;
			di[j] = from_di[i];
			di[j].index = -1;
			*di[j].file_name = TEXT('\0');
			di[j].file_name_hash = 0;
			di[j].vs_index = -1;
			di[j].start_score = 0;

			ZeroMemory(&di[j].start_time, sizeof(SYSTEMTIME));
			if (exi.date.wYear != 0 || exi.sub_type != EXTRACT_SUB_TYPE_NONE) {
				di[j].start_time.wYear = from_di[i].start_time.wYear;
			}
			if (exi.date.wMonth != 0 || exi.sub_type == EXTRACT_SUB_TYPE_MONTH || exi.sub_type == EXTRACT_SUB_TYPE_DAY) {
				di[j].start_time.wMonth = from_di[i].start_time.wMonth;
			}
			if (exi.date.wDay != 0 || exi.sub_type == EXTRACT_SUB_TYPE_DAY) {
				di[j].start_time.wDay = from_di[i].start_time.wDay;
			}
			continue;
		}
		// }[W
		di[j].sets += from_di[i].sets;
		di[j].sets_play += from_di[i].sets_play;
		di[j].legs += from_di[i].legs;
		di[j].legs_play += from_di[i].legs_play;
		di[j].ton_count += from_di[i].ton_count;
		di[j].ton00_count += from_di[i].ton00_count;
		di[j].ton40_count += from_di[i].ton40_count;
		di[j].ton80_count += from_di[i].ton80_count;
		if (di[j].high_off < from_di[i].high_off) {
			di[j].high_off = from_di[i].high_off;
		}
		if (di[j].short_game == 0 || (di[j].short_game > from_di[i].short_game && from_di[i].short_game > 0)) {
			di[j].short_game = from_di[i].short_game;
		}
		if (di[j].long_game < from_di[i].long_game) {
			di[j].long_game = from_di[i].long_game;
		}
		di[j].all_score += from_di[i].all_score;
		di[j].all_darts += from_di[i].all_darts;
		di[j].win_darts += from_di[i].win_darts;
		di[j].win_count += from_di[i].win_count;
		di[j].first9_score += from_di[i].first9_score;
		di[j].first9_darts += from_di[i].first9_darts;
		di[j].check_out_aim += from_di[i].check_out_aim;
		di[j].check_out += from_di[i].check_out;
		di[j].all_keep_legs += from_di[i].all_keep_legs;
		di[j].win_keep_legs += from_di[i].win_keep_legs;
		di[j].all_break_legs += from_di[i].all_break_legs;
		di[j].win_break_legs += from_di[i].win_break_legs;
		di[j].success_2_80 += from_di[i].success_2_80;
		di[j].failure_2_80 += from_di[i].failure_2_80;
		di[j].success_81_130 += from_di[i].success_81_130;
		di[j].failure_81_130 += from_di[i].failure_81_130;
		di[j].success_131 += from_di[i].success_131;
		di[j].failure_131 += from_di[i].failure_131;
	}
	return TRUE;
}

/*
 * data_subtract_stats - \f[^Wvl
 */
int data_subtract_stats(DATA_INFO *from_di, const int st, const int en, DATA_INFO *di, int *ret)
{
	int ret_count = 0;
	int i, j;

	if (exi.sub_type == EXTRACT_SUB_TYPE_ALL) {
		return 0;
	}
	for (i = st; i < en; i++) {
		// õ`FbN
		if (data_extract_check(&from_di[i]) == FALSE) {
			continue;
		}
		// }[WΏۂ̌
		for (j = 0; j < *ret; j++) {
			if (di[j].status == DATA_STATUS_DELETE) {
				// 폜ς
				continue;
			}
			if (data_marge_check(&di[j], &from_di[i]) == TRUE) {
				break;
			}
		}
		if (j >= *ret) {
			continue;
		}
		// Wvl
		di[j].sets -= from_di[i].sets;
		di[j].sets_play -= from_di[i].sets_play;
		di[j].legs -= from_di[i].legs;
		di[j].legs_play -= from_di[i].legs_play;
		di[j].ton_count -= from_di[i].ton_count;
		di[j].ton00_count -= from_di[i].ton00_count;
		di[j].ton40_count -= from_di[i].ton40_count;
		di[j].ton80_count -= from_di[i].ton80_count;
		di[j].all_score -= from_di[i].all_score;
		di[j].all_darts -= from_di[i].all_darts;
		di[j].win_darts -= from_di[i].win_darts;
		di[j].win_count -= from_di[i].win_count;
		di[j].first9_score -= from_di[i].first9_score;
		di[j].first9_darts -= from_di[i].first9_darts;
		di[j].check_out_aim -= from_di[i].check_out_aim;
		di[j].check_out -= from_di[i].check_out;
		di[j].all_keep_legs -= from_di[i].all_keep_legs;
		di[j].win_keep_legs -= from_di[i].win_keep_legs;
		di[j].all_break_legs -= from_di[i].all_break_legs;
		di[j].win_break_legs -= from_di[i].win_break_legs;
		di[j].success_2_80 -= from_di[i].success_2_80;
		di[j].failure_2_80 -= from_di[i].failure_2_80;
		di[j].success_81_130 -= from_di[i].success_81_130;
		di[j].failure_81_130 -= from_di[i].failure_81_130;
		di[j].success_131 -= from_di[i].success_131;
		di[j].failure_131 -= from_di[i].failure_131;
		ret_count++;
	}
	return ret_count;
}

/*
 * data_split_set - CSVf[^ݒ
 */
static BOOL data_split_set(DATA_INFO *di, const int *split_index, const int cnt, const int index)
{
	CSV_SCORE csvs;
	int csvs_index;
	TCHAR path[MAX_PATH];
	int i, j;
	int round;

	// XRAǂݍ
	ZeroMemory(&csvs, sizeof(CSV_SCORE));
	csvs.csvsi_size = di[index].legs_play * 2;
	csvs.csvsi = (CSV_SCORE_INFO *)mem_calloc(sizeof(CSV_SCORE_INFO) * csvs.csvsi_size);
	*path = TEXT('\0');
	if (file_find(csv_path, di[index].file_name, path) == FALSE || csv_score_read(path, &csvs) == FALSE) {
		csv_score_free(&csvs);
		return FALSE;
	}
	if (index < di[index].vs_index) {
		csvs_index = 0;
	} else {
		csvs_index = 1;
	}
	for (j = csvs_index; j < csvs.csvsi_count; j += 2) {
		round = 0;
		while (round <= csvs.csvsi[j].round) {
			for (i = 0; i < cnt && round <= csvs.csvsi[j].round; i++) {
				if (csvs.csvsi[j].score[round] < 0) {
					di[split_index[i]].all_score += csvs.csvsi[j].left[round - 1];
					di[split_index[i]].all_darts += csvs.csvsi[j].score[round] * -1;
					// Ton
					if (csvs.csvsi[j].left[round - 1] >= 100) {
						di[split_index[i]].ton_count++;
					}
					if (csvs.csvsi[j].left[round - 1] >= 100 && csvs.csvsi[j].left[round - 1] <= 139) {
						di[split_index[i]].ton00_count++;
					}
					if (csvs.csvsi[j].left[round - 1] >= 140 && csvs.csvsi[j].left[round - 1] <= 179) {
						di[split_index[i]].ton40_count++;
					}
					if (csvs.csvsi[j].left[round - 1] == 180) {
						di[split_index[i]].ton80_count++;
					}
					// Hi-out
					if (di[split_index[i]].high_off < csvs.csvsi[j].left[round - 1]) {
						di[split_index[i]].high_off = csvs.csvsi[j].left[round - 1];
					}
					round++;
					break;
				} else {
					if (csvs.csvsi[j].left[round] == 0) {
						round++;
						break;
					}
					di[split_index[i]].all_score += csvs.csvsi[j].score[round];
					di[split_index[i]].all_darts += 3;
					// Ton
					if (csvs.csvsi[j].score[round] >= 100) {
						di[split_index[i]].ton_count++;
					}
					if (csvs.csvsi[j].score[round] >= 100 && csvs.csvsi[j].score[round] <= 139) {
						di[split_index[i]].ton00_count++;
					}
					if (csvs.csvsi[j].score[round] >= 140 && csvs.csvsi[j].score[round] <= 179) {
						di[split_index[i]].ton40_count++;
					}
					if (csvs.csvsi[j].score[round] == 180) {
						di[split_index[i]].ton80_count++;
					}
				}
				round++;
			}
		}
	}
	csv_score_free(&csvs);
	return TRUE;
}

/*
 * data_split_data - f[^
 */
BOOL data_split_data(const HWND hWnd, const TCHAR *split_str, const int split_delete, DATA_INFO **di, int *di_count, int *di_size, const int index)
{
	DATA_INFO *tmp_di;
	TCHAR *p, *r;
	TCHAR org_name[NAME_SIZE];
	TCHAR name[NAME_SIZE];
	UINT name_hash;
	int split_index[NAME_SIZE];
	int cnt;
	int len;
	int i, j;
	BOOL split_flag = FALSE;
	BOOL ret;

	if (index < 0) {
		return FALSE;
	}
	len = lstrlen(split_str);
	cnt = 0;
	lstrcpy(org_name, (*di)[index].name);
	p = r = org_name;
	while (1) {
		if (*p == TEXT('\0') || (lstrlen(p) > len && str_cmp_ni(p, split_str, len) == 0)) {
			if (p > r) {
				if (*p == TEXT('\0') && split_flag == FALSE) {
					// 
					return TRUE;
				}
				split_flag = TRUE;
				if (split_delete == 1) {
					// f[^폜
					(*di)[index].status = DATA_STATUS_DELETE;
				}
				lstrcpyn(name, r, p - r + 1);
				name_hash = str2hash(name);
				// ɒǉς݂`FbN
				for (i = 0; i < *di_count; i++) {
					if (name_hash == (*di)[i].name_hash && (*di)[index].file_name_hash == (*di)[i].file_name_hash &&
						lstrcmpi(name, (*di)[i].name) == 0 && lstrcmpi((*di)[index].file_name, (*di)[i].file_name) == 0) {
						// ɒǉς
						j = i;
						if (hWnd != NULL) {
							data_subtract_stats(*di, j, j + 1, g_show_di, &g_show_di_count);
						}
						break;
					}
				}
				if (i >= *di_count) {
					// VKǉ
					if (*di_count + 1 > *di_size) {
						// Ċm
						*di_size += ((*di_count - *di_size) + 1 + DATA_ALLOC_SIZE);
						tmp_di = mem_calloc(sizeof(DATA_INFO) * (*di_size));
						if (*di != NULL) {
							CopyMemory(tmp_di, *di, sizeof(DATA_INFO) * (*di_count));
							mem_free(&(*di));
						}
						*di = tmp_di;
					}
					j = *di_count;
					(*di_count)++;
				}
				// {̃Rs[
				ZeroMemory(&(*di)[j], sizeof(DATA_INFO));
				lstrcpy((*di)[j].name, name);
				(*di)[j].name_hash = name_hash;
				(*di)[j].index = index;
				(*di)[j].vs_index = (*di)[index].vs_index;
				lstrcpy((*di)[j].file_name, (*di)[index].file_name);
				(*di)[j].file_name_hash = (*di)[index].file_name_hash;
				(*di)[j].start_time = (*di)[index].start_time;
				(*di)[j].start_score = (*di)[index].start_score;
				(*di)[j].sets = (*di)[index].sets;
				(*di)[j].sets_play = (*di)[index].sets_play;
				(*di)[j].legs = (*di)[index].legs;
				(*di)[j].legs_play = (*di)[index].legs_play;
				(*di)[j].all_keep_legs = (*di)[index].all_keep_legs;
				(*di)[j].win_keep_legs = (*di)[index].win_keep_legs;
				(*di)[j].all_break_legs = (*di)[index].all_break_legs;
				(*di)[j].win_break_legs = (*di)[index].win_break_legs;

				split_index[cnt] = j;
				cnt++;
			}
			if (*p == TEXT('\0')) {
				break;
			}
			p += len;
			r = p;
		} else {
			p++;
		}
	}
	ret = data_split_set(*di, split_index, cnt, index);
	if (hWnd != NULL) {
		// \XV
		for (i = 0; i < cnt; i++) {
			SendMessage(hWnd, WM_MAIN_UPDATE_ITEM, split_index[i], split_index[i] + 1);
		}
	}
	return ret;
}

/*
 * data_show_data - f[^o
 */
BOOL data_show_data(const HWND hWnd, const DATA_INFO *di)
{
	TCHAR ap_path[MAX_PATH];
	TCHAR csv_file[MAX_PATH];
	TCHAR path[MAX_PATH];
	TCHAR err_str[BUF_SIZE];
	HINSTANCE ret;

	if (*di->file_name == TEXT('\0')) {
		return FALSE;
	}
	lstrcpy(ap_path, out_data_path);
	if (show_output(hInst, hWnd, ap_path) == FALSE) {
		return FALSE;
	}
	*path = TEXT('\0');
	if (file_find(csv_path, di->file_name, path) == FALSE) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, message_get_res(IDS_STRING_OUTPUT_DATA), MB_ICONERROR);
		return FALSE;
	}
	if (str_match(TEXT("*.xls"), ap_path) == TRUE) {
		wsprintf(csv_file, TEXT("%s\\%s"), out_data_path, DATA_CSV_FILE);
		if (CopyFile(path, csv_file, FALSE) == FALSE) {
			message_get_error(GetLastError(), err_str);
			MessageBox(hWnd, err_str, message_get_res(IDS_STRING_OUTPUT_DATA), MB_ICONERROR);
			return FALSE;
		}
		ret = ShellExecute(hWnd, (LPCTSTR)NULL, (LPCTSTR)ap_path, NULL, NULL, SW_SHOWNORMAL);
	} else {
		wsprintf(csv_file, TEXT("\"%s\""), path);
		ret = ShellExecute(hWnd, (LPCTSTR)NULL, (LPCTSTR)ap_path, csv_file, NULL, SW_SHOWNORMAL);
	}
	if (ret <= (HINSTANCE)32) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, message_get_res(IDS_STRING_OUTPUT_DATA), MB_ICONERROR);
		return FALSE;
	}
	return TRUE;
}

/*
 * data_show_list - ꗗo
 */
BOOL data_show_list(const HWND hWnd, const HWND lv_wnd, const DATA_INFO *di, const int di_count)
{
	HCURSOR old_cursor;
	TCHAR ap_path[MAX_PATH];
	TCHAR csv_file[MAX_PATH];
	TCHAR err_str[BUF_SIZE];
	HINSTANCE ret;

	lstrcpy(ap_path, out_list_path);
	if (show_output(hInst, hWnd, ap_path) == FALSE) {
		return FALSE;
	}
	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	wsprintf(csv_file, TEXT("%s\\%s"), out_list_path, LIST_CSV_FILE);
	csv_write_list(lv_wnd, csv_file, di, di_count);
	SetCursor(old_cursor);
	if (str_match(TEXT("*.xls"), ap_path) == TRUE) {
		ret = ShellExecute(hWnd, (LPCTSTR)NULL, (LPCTSTR)ap_path, NULL, NULL, SW_SHOWNORMAL);
	} else {
		wsprintf(csv_file, TEXT("\"%s\\%s\""), out_list_path, LIST_CSV_FILE);
		ret = ShellExecute(hWnd, (LPCTSTR)NULL, (LPCTSTR)ap_path, csv_file, NULL, SW_SHOWNORMAL);
	}
	if (ret <= (HINSTANCE)32) {
		message_get_error(GetLastError(), err_str);
		MessageBox(hWnd, err_str, message_get_res(IDS_STRING_OUTPUT_LIST), MB_ICONERROR);
		return FALSE;
	}
	return TRUE;
}
/* End of source */
