/*
 * n01_sock
 *
 * n01_sock.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <winsock.h>
#include <process.h>

#include "..\n01_plugin.h"
#include "Memory.h"
#include "file.h"
#include "code.h"

#include "resource.h"

/* Define */
#define INI_FILE					TEXT("n01_sock.ini")
#define LOG_FILE					TEXT("n01_sock.log")
#define RESENT_PATH					TEXT("resent")

#define RECV_SIZE					4096	// Mobt@TCY

#define TIMEOUT						60		// ^CAEgb

#define WM_MAIN_RELOAD_FILE			(WM_APP + 101)
#define WM_MAIN_GET_CSV_PATH		(WM_APP + 102)
#define WM_MAIN_SOCK_ERROR			(WM_APP + 103)

#define LOG_LEVEL_DISABLE			0
#define LOG_LEVEL_STANDARD			1
#define LOG_LEVEL_ERROR				1
#define LOG_LEVEL_ALL				2

/* Global Variables */
HINSTANCE hInst;
TCHAR ini_path[MAX_PATH];
TCHAR log_path[MAX_PATH];
TCHAR resent_path[MAX_PATH];

int op_log_level;
int c_th = -1;
int c_enable;
TCHAR c_name[BUF_SIZE];
TCHAR c_server[BUF_SIZE];
unsigned long c_serveraddr;
int c_port;
int s_port;

typedef struct _THD_INFO {
	HWND hWnd;
	SOCKET soc;
} THD_INFO;

/* Local Function Prototypes */
static BOOL save_log(const TCHAR *action, const TCHAR *buf, const int level);
static BOOL save_error(const TCHAR *action, const int err_code);
static char *recv_data(const SOCKET soc, int *ret_len);
static BOOL response_check(const SOCKET soc);
static unsigned int __cdecl client_send_thread(void *ti);
static BOOL client_send_csv(void);
static unsigned int __cdecl server_recv_thread(THD_INFO *ti);
static unsigned int __cdecl server_thread(THD_INFO *ti);

/*
 * DllMain - C
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
	TCHAR work_path[MAX_PATH];
	TCHAR *p, *r;
	WORD wVersionRequested;
	WSADATA wsaData;
	int nErrorStatus;
	char host_name[BUF_SIZE];

	switch (fdwReason) {
	case DLL_PROCESS_ATTACH:
		hInst = hInstance;
		// AvP[ṼpX擾
		GetModuleFileName(hInst, work_path, MAX_PATH - 1);
		for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
			if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
				p++;
				continue;
			}
#endif	// UNICODE
			if (*p == TEXT('\\') || *p == TEXT('/')) {
				r = p;
			}
		}
		*r = TEXT('\0');
		// pX쐬
		wsprintf(ini_path, TEXT("%s\\%s"), work_path, INI_FILE);
		wsprintf(log_path, TEXT("%s\\%s"), work_path, LOG_FILE);
		wsprintf(resent_path, TEXT("%s\\%s"), work_path, RESENT_PATH);

		// WinSock̏
		wVersionRequested = MAKEWORD(1, 1);
		nErrorStatus = WSAStartup(wVersionRequested, &wsaData);
		if (nErrorStatus != 0) {
			return FALSE;
		}
		gethostname(host_name, BUF_SIZE - 1);
		op_log_level = GetPrivateProfileInt(TEXT("option"), TEXT("log"), LOG_LEVEL_STANDARD, ini_path);
		s_port = GetPrivateProfileInt(TEXT("server"), TEXT("port"), 1501, ini_path);
		c_enable = GetPrivateProfileInt(TEXT("client"), TEXT("enable"), 1, ini_path);
		GetPrivateProfileString(TEXT("client"), TEXT("name"), host_name, c_name, BUF_SIZE - 1, ini_path);
		GetPrivateProfileString(TEXT("client"), TEXT("server"), TEXT(""), c_server, BUF_SIZE - 1, ini_path);
		c_port = GetPrivateProfileInt(TEXT("client"), TEXT("port"), 1501, ini_path);

		if (op_log_level != 0) DeleteFile(log_path);
		break;

	case DLL_PROCESS_DETACH:
#ifdef _DEBUG
//		mem_debug();
#endif
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
	return TRUE;
}

/*
 * get_plugin_info - vOC擾
 */
__declspec(dllexport) BOOL CALLBACK get_plugin_info(const HWND hWnd, const int index, PLUGIN_GET_INFO *pgi)
{
	switch (index) {
	case 0:
		LoadString(hInst, IDS_STRING_TITLE, pgi->title, BUF_SIZE - 1);
		lstrcpy(pgi->func_name, TEXT("n01_sock"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;
	}
	return FALSE;
}

/*
 * n01_sock_initialize - vOC̏
 */
__declspec(dllexport) BOOL CALLBACK n01_sock_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	return (CALLTYPE_START | CALLTYPE_LEG_END);
}

/*
 * prop_proc - EBhEvV[W
 */
static BOOL CALLBACK prop_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	TCHAR buf[BUF_SIZE];

	switch (msg) {
	case WM_INITDIALOG:
		SendDlgItemMessage(hDlg, IDC_CHECK_ENABLE, BM_SETCHECK, c_enable, 0);
		SendDlgItemMessage(hDlg, IDC_EDIT_NAME, WM_SETTEXT, 0, (LPARAM)c_name);
		SendDlgItemMessage(hDlg, IDC_EDIT_SERVER, WM_SETTEXT, 0, (LPARAM)c_server);
		SetDlgItemInt(hDlg, IDC_EDIT_PORT, c_port, FALSE);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, FALSE);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			c_enable = SendDlgItemMessage(hDlg, IDC_CHECK_ENABLE, BM_GETCHECK, 0, 0);
			SendDlgItemMessage(hDlg, IDC_EDIT_NAME, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)c_name);
			SendDlgItemMessage(hDlg, IDC_EDIT_SERVER, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)c_server);
			c_port = GetDlgItemInt(hDlg, IDC_EDIT_PORT, NULL, FALSE);
			c_serveraddr = 0;

			wsprintf(buf, TEXT("%d"), op_log_level);
			WritePrivateProfileString(TEXT("option"), TEXT("log"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), c_enable);
			WritePrivateProfileString(TEXT("client"), TEXT("enable"), buf, ini_path);
			WritePrivateProfileString(TEXT("client"), TEXT("name"), c_name, ini_path);
			WritePrivateProfileString(TEXT("client"), TEXT("server"), c_server, ini_path);
			wsprintf(buf, TEXT("%d"), c_port);
			WritePrivateProfileString(TEXT("client"), TEXT("port"), buf, ini_path);
			EndDialog(hDlg, TRUE);
			break;

		case IDCANCEL:
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * n01_log_property - vOC̃vpeB
 */
__declspec(dllexport) BOOL CALLBACK n01_sock_property(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_PROP), hWnd, prop_proc, (LPARAM)0);
	return PLUGIN_SUCCEED;
}

/*
 * n01_sock
 */
__declspec(dllexport) int CALLBACK n01_sock(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR csv_path[MAX_PATH];
	TCHAR tmp_path[MAX_PATH];
	TCHAR date[BUF_SIZE];
	TCHAR time[BUF_SIZE];
	TCHAR *p;

	switch (call_type) {
	case CALLTYPE_START:
		// CSVM
		if (c_enable == 0) {
			break;
		}
		// M̃t@C݂邩`FbN
		wsprintf(find_path, TEXT("%s\\*.csv"), resent_path);
		if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
			break;
		}
		do {
			if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
				continue;
			}
			// CSVt@CM
			client_send_csv();
			break;
		} while (FindNextFile(h_find_file, &find_data) == TRUE);
		FindClose(h_find_file);
		break;

	case CALLTYPE_LEG_END:
		if (c_enable == 0) {
			break;
		}
		// CSV擾
		p = (TCHAR *)SendMessage(hWnd, WM_N01_GET_CSV, 0, (LPARAM)pei->si);
		if (p == NULL) {
			return PLUGIN_ERROR;
		}
		// t@C̐
		GetDateFormat(0, 0, &pei->si->start_time, TEXT("yyyyMMdd"), date, BUF_SIZE - 1);
		GetTimeFormat(0, 0, &pei->si->start_time, TEXT("HHmmss"), time, BUF_SIZE - 1);
		lstrcpy(csv_path, resent_path);
		lstrcat(csv_path, TEXT("\\"));
		wsprintf(csv_path + lstrlen(csv_path), TEXT("%d_%s_%s_%s"), pei->si->start_score, date, time, c_name);
		lstrcpy(tmp_path, csv_path);
		lstrcat(tmp_path, TEXT(".tmp"));
		lstrcat(csv_path, TEXT(".csv"));
		//CSVۑ
		CreateDirectory(resent_path, NULL);
		if (file_write_buf(tmp_path, p, sizeof(TCHAR) * lstrlen(p)) == FALSE) {
			save_error(TEXT("file_write_buf"), GetLastError());
			SendMessage(hWnd, WM_N01_FREE_CSV, 0, (LPARAM)p);
			return PLUGIN_ERROR;
		}
		// CSV
		SendMessage(hWnd, WM_N01_FREE_CSV, 0, (LPARAM)p);
		// t@C̈ړ
		if (CopyFile(tmp_path, csv_path, FALSE) == FALSE) {
			save_error(TEXT("CopyFile"), GetLastError());
			return PLUGIN_ERROR;
		}
		DeleteFile(tmp_path);
		// CSVt@CM
		client_send_csv();
		break;
	}
	return PLUGIN_SUCCEED;
}

/*
 * save_log - O̕ۑ
 */
static BOOL save_log(const TCHAR *action, const TCHAR *buf, const int level)
{
	TCHAR time[BUF_SIZE];
	HANDLE hFile;
	DWORD ret;

	if (op_log_level < level) {
		return TRUE;
	}
	//ۑt@CJ
	hFile = CreateFile(log_path, GENERIC_WRITE, FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == NULL || hFile == (HANDLE)-1){
		return FALSE;
	}
	SetFilePointer(hFile, 0, NULL, FILE_END);

	GetTimeFormat(0, 0, NULL, NULL, time, BUF_SIZE - 1);
	lstrcat(time, TEXT(": "));

	WriteFile(hFile, time, sizeof(TCHAR) * lstrlen(time), &ret, NULL);
	WriteFile(hFile, action, sizeof(TCHAR) * lstrlen(action), &ret, NULL);
	if (buf != NULL) {
		WriteFile(hFile, TEXT(": "), sizeof(TCHAR) * lstrlen(TEXT(": ")), &ret, NULL);
		WriteFile(hFile, buf, sizeof(TCHAR) * lstrlen(buf), &ret, NULL);
	}
	WriteFile(hFile, TEXT("\r\n"), sizeof(TCHAR) * lstrlen(TEXT("\r\n")), &ret, NULL);
	CloseHandle(hFile);
	return TRUE;
}

/*
 * save_error - G[̕ۑ
 */
static BOOL save_error(const TCHAR *action, const int err_code)
{
	TCHAR err_str[BUF_SIZE];
	TCHAR buf[BUF_SIZE];
	TCHAR *p;

	*err_str = TEXT('\0');
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, err_code, 0, err_str, BUF_SIZE - 1, NULL);
	for (p = err_str; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n'); p++);
	*p = TEXT('\0');
	wsprintf(buf, TEXT("Error(%s)"), action);
	return save_log(buf, err_str, LOG_LEVEL_ERROR);
}

/*
 * recv_data - f[^M
 */
static char *recv_data(const SOCKET soc, int *ret_len)
{
	char recv_buf[RECV_SIZE];
	char *ret_buf = NULL;
	char *tmp_buf;
	char *p;
	int recv_len;

	*ret_len = 0;
    do {
		recv_len = recv(soc, recv_buf, RECV_SIZE, 0);
		if (recv_len == SOCKET_ERROR || recv_len == 0) {
			if (recv_len == SOCKET_ERROR) save_error(TEXT("recv"), WSAGetLastError());
			mem_free(&ret_buf);
			return NULL;
		}
		tmp_buf = mem_alloc(sizeof(char) * (*ret_len + recv_len + 1));
		if (tmp_buf == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			mem_free(&ret_buf);
			return NULL;
		}
		if (ret_buf != NULL) {
			CopyMemory(tmp_buf, ret_buf, *ret_len);
		}
		CopyMemory(tmp_buf + *ret_len, recv_buf, recv_len);
		mem_free(&ret_buf);
		ret_buf = tmp_buf;
		*ret_len += recv_len;

		for (p = ret_buf + (*ret_len - recv_len); p < ret_buf + *ret_len; p++) {
			if (*p == '\r' && p + 1 < ret_buf + *ret_len && *(p + 1) == '\n') {
				break;
			}
		}
		if (p >= ret_buf + *ret_len) {
			p = NULL;
		}
	} while (p == NULL);
	*p = '\0';
	// Oɕۑ
	save_log(TEXT("R"), ret_buf, LOG_LEVEL_ALL);
    return ret_buf;
}

/*
 * response_check - X|X̃`FbN
 */
static BOOL response_check(const SOCKET soc)
{
#define OK_LEN						3		// "+OK" ̃oCg
	char *buf;
	int len;
	BOOL ret;

	if ((buf = recv_data(soc, &len)) == FALSE) {
		return FALSE;
	}
	ret = ((len < OK_LEN || *buf != '+' ||
		!(*(buf + 1) == 'O' || *(buf + 1) == 'o') ||
		!(*(buf + 2) == 'K' || *(buf + 2) == 'k')) ? FALSE : TRUE);
	mem_free(&buf);
	return ret;
}

/*
 * client_send_thread - CSVM
 */
static unsigned int __cdecl client_send_thread(void *ti)
{
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR path[MAX_PATH];
	SOCKET soc;
	struct sockaddr_in serversockaddr;
	char sendbuf[BUF_SIZE];
	char *b64str;
	char *file_buf;
	DWORD file_size;

	save_log(TEXT("Start Send Thread"), NULL, LOG_LEVEL_STANDARD);
	// \Pbg쐬
	save_log(TEXT("socket"), NULL, LOG_LEVEL_ALL);
	soc = socket(PF_INET, SOCK_STREAM, 0);
	if (soc == INVALID_SOCKET) {
		save_error(TEXT("socket"), WSAGetLastError());
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// ڑ
	save_log(TEXT("connect"), NULL, LOG_LEVEL_ALL);
	serversockaddr.sin_family = AF_INET;
	serversockaddr.sin_addr.s_addr = c_serveraddr;
	serversockaddr.sin_port = htons((unsigned short)c_port);
	ZeroMemory(serversockaddr.sin_zero, sizeof(serversockaddr.sin_zero));
	if (connect(soc, (struct sockaddr *)&serversockaddr, sizeof(serversockaddr)) == SOCKET_ERROR) {
		save_error(TEXT("connect"), WSAGetLastError());
		closesocket(soc);
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// X|X
	if (response_check(soc) == FALSE) {
		shutdown(soc, 2);
		closesocket(soc);
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// t@C
	wsprintf(find_path, TEXT("%s\\*.csv"), resent_path);
	save_log(TEXT("Find File"), find_path, LOG_LEVEL_ALL);
	if ((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE) {
		save_error(TEXT("FindFirstFile"), GetLastError());
		shutdown(soc, 2);
		closesocket(soc);
		c_th = -1;
		save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	do {
		if (find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			continue;
		}
		save_log(TEXT("Send File"), NULL, LOG_LEVEL_STANDARD);
		// t@CM
		wsprintf(sendbuf, TEXT("%s\r\n"), find_data.cFileName);
		save_log(TEXT("S"), find_data.cFileName, LOG_LEVEL_ALL);
		if (send(soc, sendbuf, strlen(sendbuf), 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			break;
		}
		if (response_check(soc) == FALSE) {
			break;
		}
		// t@Cǂݍ
		wsprintf(path, TEXT("%s\\%s"), resent_path, find_data.cFileName);
		if ((file_buf = file_read_buf(path, &file_size)) == NULL) {
			save_error(TEXT("file_read_buf"), GetLastError());
			break;
		}
		// Base64ϊ
		b64str = (char *)mem_alloc(file_size * 2 + 4);
		if (b64str == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			mem_free(&file_buf);
			break;
		}
		base64_encode(file_buf, b64str, file_size);
		mem_free(&file_buf);
		// M
		save_log(TEXT("S"), b64str, LOG_LEVEL_ALL);
		if (send(soc, b64str, strlen(b64str), 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			mem_free(&b64str);
			break;
		}
		mem_free(&b64str);
		if (send(soc, "\r\n", 2, 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			break;
		}
		if (response_check(soc) == TRUE) {
			if (file_size == file_get_size(path)) {
				// t@C폜
				save_log(TEXT("Delete"), path, LOG_LEVEL_ALL);
				if (DeleteFile(path) == FALSE) {
					save_error(TEXT("DeleteFile"), GetLastError());
				}
			}
		}
	} while (FindNextFile(h_find_file, &find_data) == TRUE);
	FindClose(h_find_file);

	save_log(TEXT("closesocket"), NULL, LOG_LEVEL_ALL);
	shutdown(soc, 2);
	closesocket(soc);
	c_th = -1;
	save_log(TEXT("End Send Thread"), NULL, LOG_LEVEL_STANDARD);
	_endthread();
	return 0;
}

/*
 * client_send_csv - CSVM
 */
static BOOL client_send_csv(void)
{
	struct hostent *serverhostent;

	if(*c_server == TEXT('\0')) {
		return FALSE;
	}
	if (c_th != -1) {
		return TRUE;
	}
	// zXg擾
	if (c_serveraddr == 0) {
		save_log(TEXT("gethostbyname"), c_server, LOG_LEVEL_STANDARD);
		c_serveraddr = inet_addr(c_server);
		if (c_serveraddr == -1) {
			serverhostent = gethostbyname(c_server);
			if (serverhostent == NULL) {
				save_error(TEXT("gethostbyname"), WSAGetLastError());
				return FALSE;
			} else {
				c_serveraddr = *((unsigned long *)((serverhostent->h_addr_list)[0]));
			}
		}
	}
	if ((c_th = _beginthread(client_send_thread, 0, NULL)) == -1) {
		save_error(TEXT("_beginthread"), GetLastError());
		return FALSE;
	}
	return TRUE;
}

/*
 * server_recv_thread - M
 */
static unsigned int __cdecl server_recv_thread(THD_INFO *ti)
{
	TCHAR data_path[MAX_PATH];
	TCHAR file_path[MAX_PATH];
	TCHAR tmp_path[MAX_PATH];
	TCHAR file_name[MAX_PATH];
	char *buf;
	char *save_buf;
	char *r;
	int size;
	BOOL ret;

	save_log(TEXT("Start Recv Thread"), NULL, LOG_LEVEL_STANDARD);
	// X|X
	save_log(TEXT("S"), TEXT("+OK"), LOG_LEVEL_ALL);
	if (send(ti->soc, "+OK\r\n", strlen("+OK\r\n"), 0) == SOCKET_ERROR) {
		save_error(TEXT("send"), WSAGetLastError());
		closesocket(ti->soc);
		mem_free(&ti);
		save_log(TEXT("End Recv Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	while (1) {
		// t@CM
		if ((buf = recv_data(ti->soc, &size)) == NULL) {
			break;
		}
		lstrcpyn(file_name, buf, MAX_PATH - 1);
		mem_free(&buf);
		save_log(TEXT("S"), TEXT("+OK"), LOG_LEVEL_ALL);
		if (send(ti->soc, "+OK\r\n", strlen("+OK\r\n"), 0) == SOCKET_ERROR) {
			save_error(TEXT("send"), WSAGetLastError());
			break;
		}
		// CSVf[^M
		save_log(TEXT("Recv File"), NULL, LOG_LEVEL_STANDARD);
		if ((buf = recv_data(ti->soc, &size)) == NULL) {
			break;
		}
		if ((save_buf = mem_alloc(size)) == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			mem_free(&buf);
			save_log(TEXT("S"), TEXT("-ERR"), LOG_LEVEL_ALL);
			send(ti->soc, "-ERR\r\n", strlen("-ERR\r\n"), 0);
			break;
		}
		// Base64fR[h
		r = base64_decode(buf, save_buf);
		mem_free(&buf);
		// ۑ擾
		SendMessage(ti->hWnd, WM_MAIN_GET_CSV_PATH, 0, (LPARAM)data_path);
		wsprintf(tmp_path, TEXT("%s\\%lX.tmp"), data_path, (long)ti);
		wsprintf(file_path, TEXT("%s\\%s"), data_path, file_name);
		// ۑ
		ret = file_write_buf(tmp_path, save_buf, r - save_buf);
		mem_free(&save_buf);
		if (ret == TRUE) {
			// t@C̈ړ
			save_log(TEXT("CopyFile"), NULL, LOG_LEVEL_ALL);
			if (CopyFile(tmp_path, file_path, FALSE) == FALSE) {
				save_error(TEXT("CopyFile"), GetLastError());
				ret = FALSE;
			}
			DeleteFile(tmp_path);
		} else {
			save_error(TEXT("file_write_buf"), GetLastError());
		}
		if (ret == FALSE) {
			save_log(TEXT("S"), TEXT("-ERR"), LOG_LEVEL_ALL);
			if (send(ti->soc, "-ERR\r\n", strlen("-ERR\r\n"), 0) == SOCKET_ERROR) {
				save_error(TEXT("send"), WSAGetLastError());
				break;
			}
		} else {
			SendMessage(ti->hWnd, WM_MAIN_RELOAD_FILE, 0, (LPARAM)file_name);
			save_log(TEXT("S"), TEXT("+OK"), LOG_LEVEL_ALL);
			if (send(ti->soc, "+OK\r\n", strlen("+OK\r\n"), 0) == SOCKET_ERROR) {
				save_error(TEXT("send"), WSAGetLastError());
				break;
			}
		}
	}
	save_log(TEXT("closesocket(Recv)"), NULL, LOG_LEVEL_ALL);
	closesocket(ti->soc);
	mem_free(&ti);
	save_log(TEXT("End Recv Thread"), NULL, LOG_LEVEL_STANDARD);
	_endthread();
	return 0;
}

/*
 * server_thread - CSVMpT[o
 */
static unsigned int __cdecl server_thread(THD_INFO *ti)
{
	struct sockaddr_in serversockaddr;
	SOCKET soc;
	SOCKET new_soc;
	struct timeval waittime;
	fd_set rdps;
	int selret;
	THD_INFO *new_ti;

	save_log(TEXT("Start Server Thread"), NULL, LOG_LEVEL_STANDARD);
	// \Pbg쐬
	save_log(TEXT("socket"), NULL, LOG_LEVEL_ALL);
	soc = socket(PF_INET, SOCK_STREAM, 0);
	if (soc == INVALID_SOCKET) {
		save_error(TEXT("socket"), WSAGetLastError());
		SendMessage(ti->hWnd, WM_MAIN_SOCK_ERROR, 0, (LPARAM)TEXT("socket"));
		mem_free(&ti);
		save_log(TEXT("End Server Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return FALSE;
	}
	save_log(TEXT("bind"), NULL, LOG_LEVEL_ALL);
	serversockaddr.sin_family = AF_INET;
	serversockaddr.sin_addr.s_addr = htonl((unsigned long)0);
	serversockaddr.sin_port = htons((unsigned short)s_port);
	ZeroMemory(serversockaddr.sin_zero, sizeof(serversockaddr.sin_zero));
	if (bind(soc, (struct sockaddr *)&serversockaddr, sizeof(serversockaddr)) == SOCKET_ERROR) {
		save_error(TEXT("bind"), WSAGetLastError());
		SendMessage(ti->hWnd, WM_MAIN_SOCK_ERROR, 0, (LPARAM)TEXT("bind"));
		closesocket(soc);
		mem_free(&ti);
		save_log(TEXT("End Server Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	save_log(TEXT("listen"), NULL, LOG_LEVEL_ALL);
	if (listen(soc, 1) == SOCKET_ERROR) {
		save_error(TEXT("listen"), WSAGetLastError());
		SendMessage(ti->hWnd, WM_MAIN_SOCK_ERROR, 0, (LPARAM)TEXT("listen"));
		closesocket(soc);
		mem_free(&ti);
		save_log(TEXT("End Server Thread"), NULL, LOG_LEVEL_STANDARD);
		_endthread();
		return 0;
	}
	// ^CAEgݒ
	waittime.tv_sec = TIMEOUT;
	waittime.tv_usec = 0;
	while (1) {
		ZeroMemory(&rdps, sizeof(fd_set));
		FD_ZERO(&rdps);
		FD_SET(soc, &rdps);
		selret = select(FD_SETSIZE, &rdps, (fd_set *)0, (fd_set *)0, &waittime);
		if (selret == SOCKET_ERROR) {
			save_error(TEXT("select"), WSAGetLastError());
			SendMessage(ti->hWnd, WM_MAIN_SOCK_ERROR, 0, (LPARAM)TEXT("select"));
			break;
		}
		if (selret == 0) {
			continue;
		}
		save_log(TEXT("accept"), NULL, LOG_LEVEL_ALL);
		if ((new_soc = accept(soc, (struct sockaddr *)0, (int *)0)) == INVALID_SOCKET) {
			save_error(TEXT("accept"), WSAGetLastError());
			continue;
		}
		// MpXbh
		new_ti = (THD_INFO *)mem_calloc(sizeof(THD_INFO));
		if (new_ti == NULL) {
			save_error(TEXT("mem_alloc"), GetLastError());
			closesocket(new_soc);
			continue;
		}
		*new_ti = *ti;
		new_ti->soc = new_soc;
		//Xbh̍쐬
		if (_beginthread(server_recv_thread, 0, new_ti) == -1) {
			save_error(TEXT("_beginthread"), GetLastError());
		}
	}
	save_log(TEXT("closesocket(Server)"), NULL, LOG_LEVEL_ALL);
	closesocket(soc);
	mem_free(&ti);
	save_log(TEXT("End Server Thread"), NULL, LOG_LEVEL_STANDARD);
	_endthread();
	return 0;
}

/*
 * server_start - T[oJn
 */
__declspec(dllexport) int CALLBACK server_start(const HWND hWnd)
{
	THD_INFO *ti;

	save_log(TEXT("Start Server"), NULL, LOG_LEVEL_STANDARD);
	ti = (THD_INFO *)mem_calloc(sizeof(THD_INFO));
	if (ti == NULL) {
		save_error(TEXT("mem_alloc"), GetLastError());
		return -1;
	}
	ti->hWnd = hWnd;
	//Xbh̍쐬
	return _beginthread(server_thread, 0, ti);
}
/* End of source */
