/*
 * n01
 *
 * score_list.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef	_INC_OLE
#include <tchar.h>

#include "general.h"
#include "Memory.h"
#include "String.h"
#include "Message.h"
#include "font.h"
#include "score_list.h"

#include "resource.h"

/* Define */
#define WINDOW_CLASS					TEXT("score_list_wnd")

#define CHAR_COUNT						17

#define D_COLOR_BACKGROUND				GetSysColor(COLOR_WINDOW)
#define D_COLOR_ODD_BACKGROUND			GetSysColor(COLOR_WINDOW)
#define D_COLOR_TEXT					GetSysColor(COLOR_WINDOWTEXT)
#define D_COLOR_HEADER_BACKGROUND		GetSysColor(COLOR_3DFACE)
#define D_COLOR_HEADER_TEXT				GetSysColor(COLOR_BTNTEXT)
#define D_COLOR_LINE					GetSysColor(COLOR_3DSHADOW)
#define D_COLOR_SEPARATE				GetSysColor(COLOR_BTNTEXT)
#define D_COLOR_TON_CIRCLE				GetSysColor(COLOR_3DSHADOW)

// zC[bZ[W
#ifndef WM_MOUSEWHEEL
#define WM_MOUSEWHEEL					0x020A
#endif
#define WHEEL_COUNT						3

/* Global Variables */
extern HINSTANCE hInst;
extern OPTION op;

static SCORE_INFO si;
static LEG_INFO leg;

typedef struct _DRAW_BUFFER {
	int view_leg;			// \bO
	BOOL half;				// n[t[h

	int max_y;				// XN[ől
	int pos_y;				// XN[{bNẌʒu
	int page_y;				// y[WTCY

	int input_left[2];		// ̓XRA̍W
	int score_left[2];		// cXRA̍W
	int score_right[2];		// cXRẢEW

	// `
	HDC draw_dc;
	HBITMAP draw_bmp;
	HBITMAP draw_ret_bmp;

	// wi`
	HDC back_dc;
	HBITMAP back_bmp;
	HBITMAP back_ret_bmp;
	int back_width;
	int back_height;
	BOOL back_redraw;
	int back_first;

	// ~`
	HDC ellipse_dc;
	HBITMAP ellipse_bmp;
	HBITMAP ellipse_ret_bmp;

	// tHg
	HFONT header_font;
	int header_height;
	HFONT score_font;
	int score_height;
	HFONT back_ret_font;
	HFONT draw_ret_font;

	// uV
	HBRUSH back_brush;
	HBRUSH odd_back_brush;
	HBRUSH header_back_brush;

	// y
	HPEN line_pen;
	HPEN separate_pen;
	HPEN separate_bold_pen;
	HPEN finish_pen;
	HPEN ton_circle_pen;
	int pen_size;

	// XRA
	SCORE_INFO *si;
} DRAW_BUFFER;

/* Local Function Prototypes */
static void set_scrollbar(const HWND hWnd, DRAW_BUFFER *bf, const SCORE_INFO *si);
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf);
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf);
static void draw_text(const HDC hdc, const TCHAR *str, const int len, const RECT *rect);
static BOOL draw_background(const DRAW_BUFFER *bf, const int first);
static BOOL draw_line(DRAW_BUFFER *bf, const SCORE_INFO *si, const RECT *rect, const int round, int *left_score);
static LRESULT CALLBACK score_list_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * set_scrollbar - XN[o[̐ݒ
 */
static void set_scrollbar(const HWND hWnd, DRAW_BUFFER *bf, const SCORE_INFO *si)
{
	SCROLLINFO sci;
	RECT rect;

	GetClientRect(hWnd, &rect);
	bf->page_y = ((rect.bottom - bf->header_height) + 1) / bf->score_height;

	// cXN[o[
	if (bf->page_y < si->leg[bf->view_leg].max_round + 1) {
		EnableScrollBar(hWnd, SB_VERT, ESB_ENABLE_BOTH);

		bf->max_y = si->leg[bf->view_leg].max_round - (bf->page_y - 1);
		bf->pos_y = (bf->pos_y < bf->max_y) ? bf->pos_y : bf->max_y;

		ZeroMemory(&sci, sizeof(SCROLLINFO));
		sci.cbSize = sizeof(SCROLLINFO);
		sci.fMask = SIF_POS | SIF_RANGE | SIF_PAGE;
		sci.nPage = bf->page_y;
		sci.nMax = si->leg[bf->view_leg].max_round;
		sci.nPos = bf->pos_y;
		SetScrollInfo(hWnd, SB_VERT, &sci, TRUE);
	} else {
		EnableScrollBar(hWnd, SB_VERT, ESB_DISABLE_BOTH);

		bf->max_y = bf->pos_y = 0;

		ZeroMemory(&sci, sizeof(SCROLLINFO));
		sci.cbSize = sizeof(SCROLLINFO);
		sci.fMask = SIF_POS | SIF_PAGE | SIF_RANGE;
		sci.nMax = 1;
		SetScrollInfo(hWnd, SB_VERT, &sci, TRUE);
	}
}

/*
 * draw_init - `̏
 */
static BOOL draw_init(const HWND hWnd, DRAW_BUFFER *bf)
{
	HDC hdc;
	HFONT ret_font;
	TEXTMETRIC tm;
	RECT rect;
	int i;

	GetClientRect(hWnd, &rect);
	hdc = GetDC(hWnd);

	// Oi
	bf->score_font = font_create(op.score_list_font, rect.right / CHAR_COUNT / ((bf->half == TRUE) ? 2 : 1), 0, FALSE, FALSE);
	ret_font = SelectObject(hdc, bf->score_font);
	GetTextMetrics(hdc, &tm);
	SelectObject(hdc, ret_font);
	bf->score_height = tm.tmHeight + tm.tmHeight / 6;
	bf->draw_bmp = CreateCompatibleBitmap(hdc, rect.right, bf->score_height);
	bf->draw_ret_bmp = SelectObject(bf->draw_dc, bf->draw_bmp);
	bf->draw_ret_font = SelectObject(bf->draw_dc, bf->score_font);

	// wi
	bf->header_font = font_create(op.score_list_font, rect.right / CHAR_COUNT / 2, 0, FALSE, FALSE);
	ret_font = SelectObject(hdc, bf->header_font);
	GetTextMetrics(hdc, &tm);
	SelectObject(hdc, ret_font);
	bf->header_height = tm.tmHeight + tm.tmHeight / 6;
	bf->back_width = rect.right;
	bf->back_height = bf->header_height + bf->score_height * 2;
	bf->back_redraw = TRUE;
	bf->back_bmp = CreateCompatibleBitmap(hdc, rect.right, bf->back_height);
	bf->back_ret_bmp = SelectObject(bf->back_dc, bf->back_bmp);
	bf->back_ret_font = SelectObject(bf->back_dc, bf->header_font);

	// ʒu̐ݒ
	i = (rect.right * 100) / CHAR_COUNT;
	bf->input_left[0] = 0;
	bf->score_left[0] = i * 3 / 100;
	bf->score_right[0] = i * 7 / 100;
	bf->input_left[1] = i * 10 / 100;
	bf->score_left[1] = i * 13 / 100;
	bf->score_right[1] = rect.right;

	// ~
	bf->ellipse_bmp = CreateCompatibleBitmap(hdc, (bf->score_left[1] - bf->input_left[1]) * 2, bf->score_height * 2);
	bf->ellipse_ret_bmp = SelectObject(bf->ellipse_dc, bf->ellipse_bmp);

	ReleaseDC(hWnd, hdc);
	return TRUE;
}

/*
 * draw_free - `̉
 */
static BOOL draw_free(const HWND hWnd, DRAW_BUFFER *bf)
{
	if (bf->draw_dc != NULL) {
		SelectObject(bf->draw_dc, bf->draw_ret_bmp);
		SelectObject(bf->draw_dc, bf->draw_ret_font);
	}
	if (bf->back_dc != NULL) {
		SelectObject(bf->back_dc, bf->back_ret_bmp);
		SelectObject(bf->back_dc, bf->back_ret_font);
	}
	if (bf->ellipse_dc != NULL) {
		SelectObject(bf->ellipse_dc, bf->ellipse_ret_bmp);
	}
	if (bf->draw_bmp != NULL) {
		DeleteObject(bf->draw_bmp);
		bf->draw_bmp = NULL;
	}
	if (bf->back_bmp != NULL) {
		DeleteObject(bf->back_bmp);
		bf->back_bmp = NULL;
	}
	if (bf->ellipse_bmp != NULL) {
		DeleteObject(bf->ellipse_bmp);
		bf->ellipse_bmp = NULL;
	}
	if (bf->score_font != NULL) {
		DeleteObject(bf->score_font);
		bf->score_font = NULL;
	}
	if (bf->header_font != NULL) {
		DeleteObject(bf->header_font);
		bf->header_font = NULL;
	}
	return TRUE;
}

/*
 * draw_text - eLXg̕`
 */
static void draw_text(const HDC hdc, const TCHAR *str, const int len, const RECT *rect)
{
	SIZE sz;
	int left, top;

	GetTextExtentPoint32(hdc, str, len, &sz);
	left = rect->left + (rect->right - rect->left - sz.cx) / 2;
	top = rect->top + (rect->bottom - rect->top - sz.cy) / 2;
	ExtTextOut(hdc, left, top, ETO_CLIPPED, rect, str, len, NULL);
}

/*
 * draw_background - wi̕`
 */
static BOOL draw_background(const DRAW_BUFFER *bf, const int first)
{
	RECT draw_rect;
	HPEN ret_pen;
	RECT rect;
	TCHAR buf[BUF_SIZE];
	int height;
	int j;

	// wi̓hԂ
	SetRect(&rect, 0, 0, bf->back_width, bf->back_height);
	FillRect(bf->back_dc, &rect, bf->back_brush);

	if (D_COLOR_BACKGROUND != D_COLOR_ODD_BACKGROUND) {
		// s̓hԂ
		height = bf->header_height + bf->score_height;
		SetRect(&draw_rect, 0, height, bf->back_width, height + bf->score_height);
		FillRect(bf->back_dc, &draw_rect, bf->odd_back_brush);
	}

	// wb_̓hԂ
	SetRect(&draw_rect, 0, 0, bf->back_width, bf->header_height);
	FillRect(bf->back_dc, &draw_rect, bf->header_back_brush);
	SetRect(&draw_rect, bf->score_right[0], 0, bf->input_left[1], bf->back_height);
	FillRect(bf->back_dc, &draw_rect, bf->header_back_brush);

	// wb_̕`
	SetTextColor(bf->back_dc, D_COLOR_HEADER_TEXT);
	SetBkColor(bf->back_dc, D_COLOR_HEADER_BACKGROUND);
	for (j = 0; j < 2; j++) {
		message_copy_res(IDS_STRING_SCORED, buf);
		SetRect(&draw_rect, bf->input_left[j], 0, bf->score_left[j], bf->header_height - 1);
		draw_text(bf->back_dc, buf, lstrlen(buf), &draw_rect);
		if (first == j) {
			message_copy_res(IDS_STRING_FIRST_MARK, buf);
			lstrcat(buf, TEXT(" "));
			message_copy_res(IDS_STRING_TO_GO, buf + lstrlen(buf));
		} else {
			message_copy_res(IDS_STRING_TO_GO, buf);
		}
		SetRect(&draw_rect, bf->score_left[j] + 1, 0, bf->score_right[j], bf->header_height - 1);
		draw_text(bf->back_dc, buf, lstrlen(buf), &draw_rect);
	}

	// ̕`
	ret_pen = SelectObject(bf->back_dc, bf->line_pen);
	// c
	for (j = 0; j < 2; j++) {
		MoveToEx(bf->back_dc, bf->score_left[j], 0, NULL);
		LineTo(bf->back_dc, bf->score_left[j], bf->back_height);
	}
	// 
	for (height = bf->header_height + bf->score_height - 1; height < bf->back_height; height = height + bf->score_height) {
		MoveToEx(bf->back_dc, 0, height, NULL);
		LineTo(bf->back_dc, bf->back_width, height);
	}
	SelectObject(bf->back_dc, ret_pen);

	// ̕`
	ret_pen = SelectObject(bf->back_dc, bf->separate_bold_pen);
	// wb_
	MoveToEx(bf->back_dc, -1, bf->header_height - 1, NULL);
	LineTo(bf->back_dc, bf->back_width, bf->header_height - 1);
	// 
	MoveToEx(bf->back_dc, bf->score_right[0] + 1, 0, NULL);
	LineTo(bf->back_dc, bf->score_right[0] + 1, bf->back_height);
	MoveToEx(bf->back_dc, bf->input_left[1] - 1, 0, NULL);
	LineTo(bf->back_dc, bf->input_left[1] - 1, bf->back_height);
	SelectObject(bf->back_dc, ret_pen);
	return TRUE;
}

/*
 * draw_line - XRAV[g̕`
 */
static BOOL draw_line(DRAW_BUFFER *bf, const SCORE_INFO *si, const RECT *rect, const int round, int *left_score)
{
	RECT draw_rect;
	HPEN ret_pen;
	HBRUSH ret_brush;
	HBRUSH ret_ellipse_brush;
	TCHAR buf[BUF_SIZE];
	int i, j;

	// wĩRs[
	BitBlt(bf->draw_dc, rect->left, 0, rect->right, bf->score_height, bf->back_dc, rect->left,
		(round / 2 == (round + 1) / 2) ? bf->header_height : (bf->header_height + bf->score_height),
		SRCCOPY);

	// ؂
	ret_pen = SelectObject(bf->draw_dc, bf->separate_pen);
	if (round % 5 == 0) {
		MoveToEx(bf->draw_dc, rect->left, bf->score_height - 1, NULL);
		LineTo(bf->draw_dc, rect->right, bf->score_height - 1);
	}
	SelectObject(bf->draw_dc, ret_pen);

	// To Go
	if (round == 0) {
		SetTextColor(bf->draw_dc, D_COLOR_TEXT);
		SetBkColor(bf->draw_dc, D_COLOR_BACKGROUND);
		for (j = 0; j < 2; j++) {
			SetRect(&draw_rect, bf->input_left[j], 0, bf->score_left[j], bf->score_height - 1);
			FillRect(bf->draw_dc, &draw_rect, bf->header_back_brush);

			_itot(si->player[j].start_score, buf, 10);
			SetRect(&draw_rect, bf->score_left[j] + 1, 0, bf->score_right[j], bf->score_height);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &draw_rect);
		}
		return TRUE;
	}

	// io[̕`
	if (si->round_limit != 0 && round > si->leg[bf->view_leg].max_round) {
		return TRUE;
	}
	SetTextColor(bf->draw_dc, D_COLOR_HEADER_TEXT);
	SetBkColor(bf->draw_dc, D_COLOR_HEADER_BACKGROUND);
	_itot(round * 3, buf, 10);
	SetRect(&draw_rect, bf->score_right[0] + 2, 0, bf->input_left[1] - 2, bf->score_height - 1);
	draw_text(bf->draw_dc, buf, lstrlen(buf), &draw_rect);

	// XRA̕`
	i = round - 1;
	if (i > si->leg[bf->view_leg].current_round) {
		return TRUE;
	}
	if (round / 2 == (round + 1) / 2) {
		SetBkColor(bf->draw_dc, D_COLOR_BACKGROUND);
		ret_brush = SelectObject(bf->draw_dc, bf->back_brush);
	} else {
		SetBkColor(bf->draw_dc, D_COLOR_ODD_BACKGROUND);
		ret_brush = SelectObject(bf->draw_dc, bf->odd_back_brush);
	}
	for (j = 0; j < 2; j++) {
		if (i == si->leg[bf->view_leg].current_round &&
			(si->leg[bf->view_leg].first != j || si->leg[bf->view_leg].current_player == j)) {
			continue;
		}
		if (si->leg[bf->view_leg].score[j][i] < 0) {
			// tBjbV{
			if (bf->pen_size == 4) {
				SetRect(&draw_rect, 0, 0, (bf->score_left[j] - bf->input_left[j]) * 2, (bf->score_height - 1) * 2);
				FillRect(bf->ellipse_dc, &draw_rect, GetCurrentObject(bf->draw_dc, OBJ_BRUSH));
				ret_ellipse_brush = SelectObject(bf->ellipse_dc, GetCurrentObject(bf->draw_dc, OBJ_BRUSH));
				ret_pen = SelectObject(bf->ellipse_dc, bf->finish_pen);
				Ellipse(bf->ellipse_dc,
					(bf->score_left[j] - bf->input_left[j] - (bf->score_height - 1)) * 2 / 2 + 2,
					2,
					(bf->score_left[j] - bf->input_left[j]) * 2 - (bf->score_left[j] - bf->input_left[j] - (bf->score_height - 1)) * 2 / 2 - 2,
					(bf->score_height - 1) * 2 - 2);
				SelectObject(bf->ellipse_dc, ret_pen);
				SelectObject(bf->ellipse_dc, ret_ellipse_brush);
				StretchBlt(bf->draw_dc, bf->input_left[j], 0, bf->score_left[j] - bf->input_left[j], bf->score_height - 1,
					bf->ellipse_dc, 0, 0, (bf->score_left[j] - bf->input_left[j]) * 2, (bf->score_height - 1) * 2, SRCCOPY);
			} else {
				ret_pen = SelectObject(bf->draw_dc, bf->finish_pen);
				Ellipse(bf->draw_dc,
					bf->input_left[j] + (bf->score_left[j] - bf->input_left[j] - (bf->score_height - 1)) / 2 + 1,
					1,
					bf->score_left[j] - (bf->score_left[j] - bf->input_left[j] - (bf->score_height - 1)) / 2 - 1,
					(bf->score_height - 1) - 1);
				SelectObject(bf->draw_dc, ret_pen);
			}

			SetTextColor(bf->draw_dc, D_COLOR_TEXT);
			_itot(si->leg[bf->view_leg].score[j][i] * -1, buf, 10);
			SetRect(&draw_rect, bf->input_left[j], 0, bf->score_left[j], bf->score_height - 1);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &draw_rect);
		} else {
			// XRA
			if (si->leg[bf->view_leg].score[j][i] >= 100) {
				// TON
				if (bf->pen_size == 4) {
					SetRect(&draw_rect, 0, 0, (bf->score_left[j] - bf->input_left[j]) * 2, (bf->score_height - 1) * 2);
					FillRect(bf->ellipse_dc, &draw_rect, GetCurrentObject(bf->draw_dc, OBJ_BRUSH));
					ret_ellipse_brush = SelectObject(bf->ellipse_dc, GetCurrentObject(bf->draw_dc, OBJ_BRUSH));
					ret_pen = SelectObject(bf->ellipse_dc, bf->ton_circle_pen);
					Ellipse(bf->ellipse_dc, 2, 2,
						(bf->score_left[j] - bf->input_left[j]) * 2 - 2, (bf->score_height - 1) * 2 - 2);
					SelectObject(bf->ellipse_dc, ret_pen);
					SelectObject(bf->ellipse_dc, ret_ellipse_brush);
					StretchBlt(bf->draw_dc, bf->input_left[j], 0, bf->score_left[j] - bf->input_left[j], bf->score_height - 1,
						bf->ellipse_dc, 0, 0, (bf->score_left[j] - bf->input_left[j]) * 2, (bf->score_height - 1) * 2, SRCCOPY);
				} else {
					ret_pen = SelectObject(bf->draw_dc, bf->ton_circle_pen);
					Ellipse(bf->draw_dc,
						bf->input_left[j] + 1, 1,
						bf->score_left[j] - 1, (bf->score_height - 1) - 1);
					SelectObject(bf->draw_dc, ret_pen);
				}
			}
			SetTextColor(bf->draw_dc, D_COLOR_TEXT);
			_itot(si->leg[bf->view_leg].score[j][i], buf, 10);
			SetRect(&draw_rect, bf->input_left[j], 0, bf->score_left[j], bf->score_height - 1);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &draw_rect);

			// cXRA
			left_score[j] -= si->leg[bf->view_leg].score[j][i];
			SetTextColor(bf->draw_dc, D_COLOR_TEXT);
			_itot(left_score[j], buf, 10);
			SetRect(&draw_rect, bf->score_left[j] + 1, 0, bf->score_right[j], bf->score_height - 1);
			draw_text(bf->draw_dc, buf, lstrlen(buf), &draw_rect);
		}
	}
	SelectObject(bf->draw_dc, ret_brush);
	return TRUE;
}

/*
 * score_list_proc - XRAꗗEBhEvV[W
 */
static LRESULT CALLBACK score_list_proc(const HWND hWnd, const UINT msg, WPARAM wParam, LPARAM lParam)
{
	DRAW_BUFFER *bf;
	HDC hdc;
	PAINTSTRUCT ps;
	RECT rect;
	TCHAR buf[BUF_SIZE];
	int left_score[2];
	int i, j;
	int y;

	switch (msg) {
	case WM_CREATE:
		// 
		bf = (DRAW_BUFFER *)mem_calloc(sizeof(DRAW_BUFFER));
		if (bf == NULL) {
			message_get_error(GetLastError(), buf);
			MessageBox(hWnd, buf, APP_NAME, MB_ICONERROR);
			return -1;
		}
		bf->si = (SCORE_INFO *)((CREATESTRUCT *)lParam)->lpCreateParams;
		if (bf->si == NULL) {
			bf->si = &si;
			bf->si->leg = &leg;
		}
		SetWindowLong(hWnd, GWL_USERDATA, (LONG)bf);

		// `p̏
		hdc = GetDC(hWnd);
		bf->draw_dc = CreateCompatibleDC(hdc);
		bf->back_dc = CreateCompatibleDC(hdc);
		bf->ellipse_dc = CreateCompatibleDC(hdc);
		ReleaseDC(hWnd, hdc);
		if (GetDeviceCaps(bf->draw_dc, BITSPIXEL) >= 16) {
			SetStretchBltMode(bf->draw_dc, HALFTONE);
			SetBrushOrgEx(bf->draw_dc, 0, 0, NULL);
			bf->pen_size = 4;
		} else {
			bf->pen_size = 2;
		}
		bf->half = TRUE;
		draw_init(hWnd, bf);
		SetBkMode(bf->draw_dc, TRANSPARENT);
		bf->back_brush = CreateSolidBrush(D_COLOR_BACKGROUND);
		bf->odd_back_brush = CreateSolidBrush(D_COLOR_ODD_BACKGROUND);
		bf->header_back_brush = CreateSolidBrush(D_COLOR_HEADER_BACKGROUND);
		bf->line_pen = CreatePen(PS_SOLID, 1, D_COLOR_LINE);
		bf->separate_pen = CreatePen(PS_SOLID, 1, D_COLOR_SEPARATE);
		bf->separate_bold_pen = CreatePen(PS_SOLID, 2, D_COLOR_SEPARATE);
		bf->finish_pen = CreatePen(PS_SOLID, 2, D_COLOR_TEXT);
		bf->ton_circle_pen = CreatePen(PS_SOLID, 2, D_COLOR_TON_CIRCLE);

		// IMEOFFɐݒ
		ImmAssociateContext(hWnd, (HIMC)NULL);
		// XN[o[̐ݒ
		set_scrollbar(hWnd, bf, bf->si);
		break;

	case WM_CLOSE:
		// EBhE
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		// EBhE̔j
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf != NULL) {
			draw_free(hWnd, bf);
			if (bf->draw_dc != NULL) {
				DeleteDC(bf->draw_dc);
				bf->draw_dc = NULL;
			}
			if (bf->back_dc != NULL) {
				DeleteDC(bf->back_dc);
				bf->back_dc = NULL;
			}
			if (bf->ellipse_dc != NULL) {
				DeleteDC(bf->ellipse_dc);
				bf->ellipse_dc = NULL;
			}
			DeleteObject(bf->back_brush);
			DeleteObject(bf->odd_back_brush);
			DeleteObject(bf->header_back_brush);
			DeleteObject(bf->line_pen);
			DeleteObject(bf->separate_pen);
			DeleteObject(bf->separate_bold_pen);
			DeleteObject(bf->finish_pen);
			DeleteObject(bf->ton_circle_pen);
			mem_free(&bf);
		}
		return DefWindowProc(hWnd, msg, wParam, lParam);

	case WM_SETFOCUS:
		break;

	case WM_SIZE:
		// EBhETCYύX
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		set_scrollbar(hWnd, bf, bf->si);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_PAINT:
		// `
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		hdc = BeginPaint(hWnd, &ps);
		if (bf->back_redraw == TRUE || bf->back_first != bf->si->leg[bf->view_leg].first) {
			bf->back_redraw = FALSE;
			bf->back_first = bf->si->leg[bf->view_leg].first;
			// wi̕`
			draw_background(bf, bf->back_first);
		}
		if (ps.rcPaint.top < bf->header_height) {
			// wb_[̃Rs[
			BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right, bf->header_height,
				bf->back_dc, ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);
			ps.rcPaint.top = bf->header_height;
		}
		// XRAV[g̕`
		y = ((ps.rcPaint.top - bf->header_height) / bf->score_height) * bf->score_height + bf->header_height;
		j = bf->pos_y + (y - bf->header_height) / bf->score_height;
		left_score[0] = bf->si->player[0].start_score;
		left_score[1] = bf->si->player[1].start_score;
		for (i = 0; i < j - 1 && i < bf->si->leg[bf->view_leg].current_round; i++) {
			left_score[0] -= bf->si->leg[bf->view_leg].score[0][i];
			left_score[1] -= bf->si->leg[bf->view_leg].score[1][i];
		}
		for (i = j; y < ps.rcPaint.bottom; y += bf->score_height, i++) {
			if (bf->si->round_limit != 0 && i > bf->si->leg[bf->view_leg].max_round) {
				SetRect(&rect, ps.rcPaint.left, y, ps.rcPaint.right, ps.rcPaint.bottom);
				FillRect(hdc, &rect, bf->header_back_brush);
				break;
			}
			draw_line(bf, bf->si, &ps.rcPaint, i, left_score);
			BitBlt(hdc, ps.rcPaint.left, y, ps.rcPaint.right, bf->score_height,
				bf->draw_dc, ps.rcPaint.left, 0, SRCCOPY);
		}
		EndPaint(hWnd, &ps);
		break;

	case WM_ERASEBKGND:
		// wiȂ
		return TRUE;

	case WM_VSCROLL:
		// cXN[
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		GetClientRect(hWnd, &rect);
		rect.top += bf->header_height;
		i = bf->pos_y;
		switch ((int)LOWORD(wParam)) {
		case SB_TOP:
			bf->pos_y = 0;
			break;

		case SB_BOTTOM:
			bf->pos_y = bf->max_y;
			break;

		case SB_LINEDOWN:
			bf->pos_y = (bf->pos_y < bf->max_y) ? bf->pos_y + 1 : bf->max_y;
			break;

		case SB_LINEUP:
			bf->pos_y = (bf->pos_y > 0) ? bf->pos_y - 1 : 0;
			break;

		case SB_PAGEDOWN:
			bf->pos_y = (bf->pos_y + (bf->page_y - 1) < bf->max_y) ?
				bf->pos_y + (bf->page_y - 1) : bf->max_y;
			break;

		case SB_PAGEUP:
			bf->pos_y = (bf->pos_y - (bf->page_y - 1) > 0) ?
				bf->pos_y - (bf->page_y - 1) : 0;
			break;

		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			{
				SCROLLINFO sci;

				ZeroMemory(&sci, sizeof(SCROLLINFO));
				sci.cbSize = sizeof(SCROLLINFO);
				sci.fMask = SIF_ALL;
				GetScrollInfo(hWnd, SB_VERT, &sci);
				bf->pos_y = sci.nTrackPos;
			}
			break;
		}
		SetScrollPos(hWnd, SB_VERT, bf->pos_y, TRUE);
		ScrollWindowEx(hWnd, 0, (i - bf->pos_y) * bf->score_height, NULL, &rect, NULL, NULL, SW_INVALIDATE | SW_ERASE);
		break;

	case WM_MOUSEWHEEL:
		// }EXzC[
		for (i = 0; i < WHEEL_COUNT; i++) {
			SendMessage(hWnd, WM_VSCROLL, ((short)HIWORD(wParam) > 0) ? SB_LINEUP : SB_LINEDOWN, 0);
		}
		break;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		SetFocus(hWnd);
		break;

	case WM_KEYDOWN:
		// L[{[h
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		switch (wParam) {
		case VK_HOME:
			SendMessage(hWnd, WM_VSCROLL, SB_TOP, 0);
			break;

		case VK_END:
			SendMessage(hWnd, WM_VSCROLL, SB_BOTTOM, 0);
			break;

		case VK_PRIOR:
			SendMessage(hWnd, WM_VSCROLL, SB_PAGEUP, 0);
			break;

		case VK_NEXT:
			SendMessage(hWnd, WM_VSCROLL, SB_PAGEDOWN, 0);
			break;

		case VK_UP:
			SendMessage(hWnd, WM_VSCROLL, SB_LINEUP, 0);
			break;

		case VK_DOWN:
			SendMessage(hWnd, WM_VSCROLL, SB_LINEDOWN, 0);
			break;
		}
		break;

	case WM_GETDLGCODE:
		// J[\L[Cxg擾
		return DLGC_WANTARROWS;

	case WM_SCORE_SET_INFO:
		// \XRAݒ
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		bf->si = (SCORE_INFO *)lParam;
		break;

	case WM_SCORE_SHOW_LEG:
		// \bO̐ݒ
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		if ((int)wParam < 0) {
			break;
		}
		bf->view_leg = wParam;
		bf->pos_y = 0;
		set_scrollbar(hWnd, bf, bf->si);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_SCORE_REDRAW:
		// XRA̍ĕ`
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		if (wParam == 0) {
			InvalidateRect(hWnd, NULL, FALSE);
		} else {
			GetClientRect(hWnd, &rect);
			i = bf->header_height +
				((bf->si->leg[bf->view_leg].current_round + 1) - bf->pos_y) * bf->score_height +
				bf->score_height;
			SetRect(&rect,
				0, bf->header_height + wParam * bf->score_height,
				rect.right, (i > rect.bottom) ? rect.bottom : i);
			InvalidateRect(hWnd, &rect, FALSE);
		}
		UpdateWindow(hWnd);
		break;

	case WM_SCORE_REFRESH:
		// XRA̍XV
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_SCORE_SET_HALF_HEIGHT:
		// XRAV[g̍𔼕ɐݒ (XRA)
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		bf->half = wParam;
		draw_free(hWnd, bf);
		draw_init(hWnd, bf);
		set_scrollbar(hWnd, bf, bf->si);
		SendMessage(hWnd, WM_SCORE_REDRAW, 0, 0);
		break;

	case WM_SCORE_GET_BUFFER:
		// `̎擾
		bf = (DRAW_BUFFER *)GetWindowLong(hWnd, GWL_USERDATA);
		if (bf == NULL) {
			break;
		}
		return (LRESULT)bf;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

/*
 * score_list_regist - EBhENX̓o^
 */
BOOL score_list_regist(const HINSTANCE hInstance)
{
	WNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)score_list_proc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = WINDOW_CLASS;
	// EBhENX̓o^
	return RegisterClass(&wc);
}

/*
 * score_list_create - EBhE̍쐬
 */
HWND score_list_create(const HINSTANCE hInstance, const HWND pWnd, int id, SCORE_INFO *si)
{
	HWND hWnd;

	// EBhE̍쐬
	hWnd = CreateWindow(WINDOW_CLASS,
		TEXT(""),
		WS_BORDER | WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_VSCROLL,
		0, 0, 100, 100, pWnd, (HMENU)id, hInstance, si);
	return hWnd;
}
/* End of source */
