/*
 * n01_stats
 *
 * view.c
 *
 * Copyright (C) 1996-2006 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <stdio.h>
#include <commctrl.h>

#include "General.h"
#include "Message.h"
#include "string.h"
#include "listview.h"
#include "file.h"
#include "csv.h"
#include "data.h"
#include "index.h"
#include "view.h"
#include "player.h"

#include "resource.h"

/* Define */
#define ABS(n)							((n < 0) ? (n * -1) : n)

/* Global Variables */
static int v_di_index;

extern HINSTANCE hInst;
extern HWND g_main_wnd;
extern OPTION op;
extern TCHAR index_file[];
extern TCHAR csv_path[];

extern DATA_INFO *g_di;
extern int g_di_count;
extern DATA_INFO *g_show_di;

/* Local Function Prototypes */
static BOOL option_notify_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static BOOL get_listview_disp_item(const HWND hListView, LV_DISPINFO *lv_disp);
static int set_listview_data(const HWND hListView, const int i, const int di_index);
static int CALLBACK compare_func(LPARAM lParam1, LPARAM lParam2, LPARAM colum);
static BOOL CALLBACK view_data_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
static int set_listview_stats(const HWND hListView, TCHAR *title, TCHAR *value);
static void show_listview_stats(const HWND hDlg, const DATA_INFO *di);
static BOOL CALLBACK view_general_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * option_notify_proc - vpeBV[g̃Cxg̒ʒm
 */
static BOOL option_notify_proc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	PSHNOTIFY *pshn = (PSHNOTIFY FAR *)lParam;
	NMHDR *lpnmhdr = (NMHDR FAR *)&pshn->hdr;

	switch (lpnmhdr->code) {
	case PSN_APPLY:				// OK
		SendMessage(hDlg, WM_COMMAND, IDOK, 0);
		break;

	case PSN_SETACTIVE:
	case PSN_KILLACTIVE:
	case PSN_QUERYCANCEL:
	case PSN_RESET:
	case PSN_HELP:
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * get_listview_disp_item - Xgr[ɕ\ACe̐ݒ
 */
static BOOL get_listview_disp_item(const HWND hListView, LV_DISPINFO *lv_disp)
{
	DATA_INFO *di;

	di = &g_di[lv_disp->item.lParam];
	if (di == NULL) {
		return FALSE;
	}
	// eLXg
	if (lv_disp->item.mask & LVIF_TEXT) {

		switch (lv_disp->item.iSubItem) {
		case 0:
			data_to_value(HDTP_DATE, di, lv_disp->item.pszText);
			break;
		case 1:
			data_to_value(HDTP_START_SCORE, di, lv_disp->item.pszText);
			break;
		case 2:
			if (di->index >= 0) {
				data_to_value(HDTP_NAME, &g_di[di->index], lv_disp->item.pszText);
			} else {
				data_to_value(HDTP_NAME, di, lv_disp->item.pszText);
			}
			break;
		case 3:
			data_to_value(HDTP_VS_NAME, di, lv_disp->item.pszText);
			break;
		case 4:
			data_to_value(HDTP_LEGS, di, lv_disp->item.pszText);
			break;
		}
	}
	return TRUE;
}

/*
 * set_listview_data - ACe̒ǉ
 */
static int set_listview_data(const HWND hListView, const int i, const int di_index)
{
	LV_ITEM lvi;

	lvi.mask = LVIF_TEXT | LVIF_PARAM;
	lvi.iItem = (i < 0) ? ListView_GetItemCount(hListView) : i;
	lvi.iSubItem = 0;
	lvi.pszText = LPSTR_TEXTCALLBACK;
	lvi.cchTextMax = 0;
	lvi.iImage = 0;
/*	if (di_index < g_di[di_index].vs_index) {
		lvi.lParam = di_index;
	} else {
		lvi.lParam = g_di[di_index].vs_index;
	}*/
	lvi.lParam = di_index;
	return ListView_InsertItem(hListView, &lvi);
}

/*
 * compare_func - \[gpr
 */
static int CALLBACK compare_func(LPARAM lParam1, LPARAM lParam2, LPARAM colum)
{
	DATA_INFO *di1 = &g_di[lParam1];
	DATA_INFO *di2 = &g_di[lParam2];
	TCHAR *p1, *p2;
	TCHAR buf1[BUF_SIZE], buf2[BUF_SIZE];
	FILETIME ft1, ft2;
	int order;
	int header;
	int ret;

	// \[g
	order = (colum < 0) ? 1 : 0;
	header = ABS(colum) - 1;

	switch (header) {
	case 0:
		SystemTimeToFileTime(&di1->start_time, &ft1);
		SystemTimeToFileTime(&di2->start_time, &ft2);
		ret = CompareFileTime(&ft1, &ft2);
		break;
	case 1:
		ret = di1->start_score - di2->start_score;
		break;
	case 2:
		if (di1->index >= 0) {
			p1 = g_di[di1->index].name;
		} else {
			p1 = di1->name;
		}
		if (di2->index >= 0) {
			p2 = g_di[di2->index].name;
		} else {
			p2 = di2->name;
		}
		ret = lstrcmpi(p1, p2);
		break;
	case 3:
		if (di1->vs_index >= 0) {
			p1 = g_di[di1->vs_index].name;
		} else {
			p1 = TEXT("");
		}
		if (di2->vs_index >= 0) {
			p2 = g_di[di2->vs_index].name;
		} else {
			p2 = TEXT("");
		}
		ret = lstrcmpi(p1, p2);
		break;
	case 4:
		if (di1->vs_index >= 0) {
			wsprintf(buf1, TEXT("%d-%d"), di1->legs, g_di[di1->vs_index].legs);
		} else {
			*buf1 = TEXT('\0');
		}
		if (di2->vs_index >= 0) {
			wsprintf(buf2, TEXT("%d-%d"), di2->legs, g_di[di2->vs_index].legs);
		} else {
			*buf2 = TEXT('\0');
		}
		ret = lstrcmp(buf1, buf2);
		break;
	}
	return (((ret < 0 && order == 1) || (ret > 0 && order == 0)) ? 1 : -1);
}

/*
 * view_data_proc - EBhEvV[W
 */
static BOOL CALLBACK view_data_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static int colum;
	HCURSOR old_cursor;
	LV_COLUMN lvc;
	DATA_INFO *di;
	int di_index;
	int i, j;
	BOOL enable;

	switch (msg) {
	case WM_INITDIALOG:
		SetWindowLong(hDlg, GWL_USERDATA, (LPARAM)v_di_index);
		di = &g_show_di[v_di_index];

		SendDlgItemMessage(hDlg, IDC_LIST_STATS, LVM_SETEXTENDEDLISTVIEWSTYLE, 0,
			LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT |
			SendDlgItemMessage(hDlg, IDC_LIST_STATS, LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0));

		lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;

		lvc.fmt = LVCFMT_LEFT;
		lvc.cx = 120;
		lvc.pszText = data_get_title(HDTP_DATE);
		lvc.iSubItem = 0;
		ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

		lvc.fmt = LVCFMT_RIGHT;
		lvc.cx = 50;
		lvc.pszText = data_get_title(HDTP_START_SCORE);
		lvc.iSubItem = 1;
		ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

		lvc.fmt = LVCFMT_LEFT;
		lvc.cx = 100;
		lvc.pszText = data_get_title(HDTP_NAME);
		lvc.iSubItem = 2;
		ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

		lvc.fmt = LVCFMT_LEFT;
		lvc.cx = 100;
		lvc.pszText = data_get_title(HDTP_VS_NAME);
		lvc.iSubItem = 3;
		ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

		lvc.fmt = LVCFMT_LEFT;
		lvc.cx = 40;
		lvc.pszText = data_get_title(HDTP_LEGS);
		lvc.iSubItem = 4;
		ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

		old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
		for (i = 0, j = 0; i < g_di_count; i++) {
			if (data_extract_check(&g_di[i]) == FALSE ||
				data_marge_check(di, &g_di[i]) == FALSE) {
				continue;
			}
			set_listview_data(GetDlgItem(hDlg, IDC_LIST_STATS), j++, i);
		}
		colum = 1;
		ListView_SortItems(GetDlgItem(hDlg, IDC_LIST_STATS), compare_func, colum);
		ListView_SetItemState(GetDlgItem(hDlg, IDC_LIST_STATS), 0, LVIS_FOCUSED, LVIS_FOCUSED);
		SendMessage(hDlg, WM_LV_EVENT, LVN_ITEMCHANGED, 0);
		SetCursor(old_cursor);
		break;

	case WM_DESTROY:
		break;

	case WM_NOTIFY:
		if (((NMHDR *)lParam)->hwndFrom == GetWindow(GetDlgItem(hDlg, IDC_LIST_STATS), GW_CHILD)) {
			HD_NOTIFY *hdn = (HD_NOTIFY *)lParam;
			if (hdn->hdr.code == HDN_ITEMCLICK) {
				colum = (ABS(colum) == (hdn->iItem + 1)) ? (colum * -1) : (hdn->iItem + 1);
				ListView_SortItems(GetDlgItem(hDlg, IDC_LIST_STATS), compare_func, colum);
			}
			break;
		}
		if (((NMHDR *)lParam)->hwndFrom == GetDlgItem(hDlg, IDC_LIST_STATS)) {
			return listview_notify_proc(hDlg, GetDlgItem(hDlg, IDC_LIST_STATS), lParam);
		}
		return option_notify_proc(hDlg, msg, wParam, lParam);

	case WM_LV_EVENT:
		switch (wParam) {
		case LVN_GETDISPINFO:
			get_listview_disp_item(GetDlgItem(hDlg, IDC_LIST_STATS), (NMLVDISPINFO *)lParam);
			break;

		case LVN_ITEMACTIVATE:
			SendMessage(hDlg, WM_COMMAND, IDC_BUTTON_VIEW, 0);
			break;

		case LVN_ITEMCHANGED:
			enable = (ListView_GetSelectedCount(GetDlgItem(hDlg, IDC_LIST_STATS)) <= 0) ? FALSE : TRUE;
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_VIEW), enable);
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_DATA_OUT), enable);
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_BUTTON_VIEW:
			// \
			if ((i = ListView_GetNextItem(GetDlgItem(hDlg, IDC_LIST_STATS), -1, LVNI_SELECTED | LVNI_FOCUSED)) == -1) {
				break;
			}
			di_index = listview_get_lparam(GetDlgItem(hDlg, IDC_LIST_STATS), i);
			if (show_player(hInst, hDlg, di_index) == TRUE) {
				InvalidateRect(GetDlgItem(hDlg, IDC_LIST_STATS), NULL, FALSE);
				UpdateWindow(GetDlgItem(hDlg, IDC_LIST_STATS));
			}
			break;

		case IDC_BUTTON_DATA_OUT:
			// f[^o
			if ((i = ListView_GetNextItem(GetDlgItem(hDlg, IDC_LIST_STATS), -1, LVNI_SELECTED | LVNI_FOCUSED)) == -1) {
				break;
			}
			data_show_data(hDlg, &g_di[listview_get_lparam(GetDlgItem(hDlg, IDC_LIST_STATS), i)]);
			break;

		case IDOK:
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * set_listview_stats - ACe̒ǉ
 */
static int set_listview_stats(const HWND hListView, TCHAR *title, TCHAR *value)
{
	LV_ITEM lvi;
	int i;

	lvi.mask = LVIF_TEXT | LVIF_PARAM;
	lvi.iItem = ListView_GetItemCount(hListView);
	lvi.iSubItem = 0;
	lvi.pszText = title;
	lvi.cchTextMax = BUF_SIZE - 1;
	lvi.iImage = 0;
	lvi.lParam = 0;
	i = ListView_InsertItem(hListView, &lvi);
	ListView_SetItemText(hListView, i, 1, value);
	return i;
}

/*
 * show_listview_stats - v̕\
 */
static void show_listview_stats(const HWND hDlg, const DATA_INFO *di)
{
	LV_COLUMN lvc;
	TCHAR buf[BUF_SIZE];

	// ACe̍폜
	ListView_DeleteAllItems(GetDlgItem(hDlg, IDC_LIST_STATS));
	// J̍폜
	while (Header_GetItemCount(ListView_GetHeader(GetDlgItem(hDlg, IDC_LIST_STATS))) > 0) {
		ListView_DeleteColumn(GetDlgItem(hDlg, IDC_LIST_STATS), 0);
	}

	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = 150;
	lvc.pszText = message_get_res(IDS_STRING_VIEW_CONTENT);
	lvc.iSubItem = 0;
	ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = 250;
	lvc.pszText = message_get_res(IDS_STRING_VIEW_VALUE);
	lvc.iSubItem = 1;
	ListView_InsertColumn(GetDlgItem(hDlg, IDC_LIST_STATS), lvc.iSubItem, &lvc);

	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SETS_PLAYED), data_to_value(HDTP_SETS_PLAYED, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SETS_WON), data_to_value(HDTP_SETS_WON, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_LEGS_PLAYED), data_to_value(HDTP_LEGS_PLAYED, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_LEGS_WON), data_to_value(HDTP_LEGS_WON, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_TON), data_to_value(HDTP_TON, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_TON00), data_to_value(HDTP_TON00, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_TON40), data_to_value(HDTP_TON40, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_TON80), data_to_value(HDTP_TON80, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_HIGH_OFF), data_to_value(HDTP_HIGH_OFF, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SHORT_GAME), data_to_value(HDTP_SHORT_GAME, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_LONG_GAME), data_to_value(HDTP_LONG_GAME, di, buf));

	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_SET), data_to_value(HDTP_AVG_SET, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_LEG), data_to_value(HDTP_AVG_LEG, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_DARTS), data_to_value(HDTP_AVG_DARTS, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_SCORE), data_to_value(HDTP_AVG_SCORE, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_FIRST9), data_to_value(HDTP_AVG_FIRST9, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_CHECK_OUT_VIEW), data_to_value(HDTP_AVG_CHECK_OUT_VIEW, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_KEEP_LEGS_VIEW), data_to_value(HDTP_AVG_KEEP_LEGS_VIEW, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_AVG_BREAK_LEGS_VIEW), data_to_value(HDTP_AVG_BREAK_LEGS_VIEW, di, buf));

	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SUCCESS_2_80), data_to_value(HDTP_SUCCESS_2_80, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_FAILURE_2_80), data_to_value(HDTP_FAILURE_2_80, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SUCCESS_81_130), data_to_value(HDTP_SUCCESS_81_130, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_FAILURE_81_130), data_to_value(HDTP_FAILURE_81_130, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_SUCCESS_131), data_to_value(HDTP_SUCCESS_131, di, buf));
	set_listview_stats(GetDlgItem(hDlg, IDC_LIST_STATS),
		data_get_title(HDTP_FAILURE_131), data_to_value(HDTP_FAILURE_131, di, buf));
}

/*
 * view_general_proc - EBhEvV[W
 */
static BOOL CALLBACK view_general_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	DATA_INFO *di;
	TCHAR buf[BUF_SIZE];
	UINT name_hash;
	int di_index;
	int i;

	switch (msg) {
	case WM_INITDIALOG:
		SetWindowLong(hDlg, GWL_USERDATA, (LPARAM)v_di_index);
		di = &g_show_di[v_di_index];

		SendDlgItemMessage(hDlg, IDC_EDIT_NAME, WM_SETTEXT, 0, (LPARAM)di->name);
		SendDlgItemMessage(hDlg, IDC_EDIT_NAME, EM_SETMODIFY, FALSE, 0);
		// \Ώۂ`FbN
		for (i = 0; i < g_di_count; i++) {
			if (data_extract_check(&g_di[i]) == FALSE ||
				data_marge_check(di, &g_di[i]) == FALSE) {
				continue;
			}
			break;
		}
		if (i >= g_di_count) {
			SendDlgItemMessage(hDlg, IDC_EDIT_NAME, EM_SETREADONLY, TRUE, 0);
		}

		if (di->start_time.wYear <= 0) {
			lstrcpy(buf, INDEFINITE_CHAR);
		}else if (di->start_time.wMonth <= 0) {
			wsprintf(buf, TEXT("%d"), di->start_time.wYear);
		}else if (di->start_time.wDay <= 0) {
			wsprintf(buf, TEXT("%d/%02d"), di->start_time.wYear, di->start_time.wMonth);
		}else if (di->start_time.wHour <= 0) {
			GetDateFormat(0, 0, &di->start_time, NULL, buf, BUF_SIZE - 1);
		}
		SendDlgItemMessage(hDlg, IDC_EDIT_DATE, WM_SETTEXT, 0, (LPARAM)buf);

		SendDlgItemMessage(hDlg, IDC_LIST_STATS, LVM_SETEXTENDEDLISTVIEWSTYLE, 0,
			LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT |
			SendDlgItemMessage(hDlg, IDC_LIST_STATS, LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0));

		show_listview_stats(hDlg, di);
		ListView_SetItemState(GetDlgItem(hDlg, IDC_LIST_STATS), 0, LVIS_FOCUSED, LVIS_FOCUSED);
		break;

	case WM_DESTROY:
		break;

	case WM_NOTIFY:
		return option_notify_proc(hDlg, msg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			if (SendDlgItemMessage(hDlg, IDC_EDIT_NAME, EM_GETMODIFY, 0, 0) == FALSE) {
				// XV
				break;
			}
			// f[^XV
			di_index = GetWindowLong(hDlg, GWL_USERDATA);
			SendDlgItemMessage(hDlg, IDC_EDIT_NAME, WM_GETTEXT, NAME_SIZE - 1, (LPARAM)buf);
			name_hash = str2hash(buf);
			for (i = 0; i < g_di_count; i++) {
				if (data_extract_check(&g_di[i]) == FALSE ||
					data_marge_check(&g_show_di[di_index], &g_di[i]) == FALSE) {
					continue;
				}
				lstrcpy(g_di[i].name, buf);
				g_di[i].name_hash = name_hash;
				if (op.csv_update != 0 && g_di[i].index == -1) {
					// CSVXV
					if (i < g_di[i].vs_index) {
						csv_write_header(csv_path, g_di[i].file_name, &g_di[i]);
					} else {
						csv_write_header(csv_path, g_di[g_di[i].vs_index].file_name, &g_di[g_di[i].vs_index]);
					}
				}
			}
			// \f[^XV
			lstrcpy(g_show_di[di_index].name, buf);
			g_show_di[di_index].name_hash = name_hash;

			index_save(index_file, g_di, 0, g_di_count);
			SendMessage(g_main_wnd, WM_MAIN_REFRESH, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * show_view - \
 */
BOOL show_view(const HINSTANCE hInst, const HWND hWnd, int di_index)
{
#define sizeof_PROPSHEETHEADER		40	// ÂRRg[΍
#define PROP_CNT_OPTION				2
	PROPSHEETPAGE psp;
	PROPSHEETHEADER psh;
	HPROPSHEETPAGE hpsp[PROP_CNT_OPTION];

	// \f[^̃Rs[
	v_di_index = di_index;

	psp.dwSize = sizeof(PROPSHEETPAGE);
	psp.dwFlags = PSP_DEFAULT;
	psp.hInstance = hInst;

	// S
	psp.pszTemplate = MAKEINTRESOURCE(IDD_DIALOG_VIEW_GENERAL);
	psp.pfnDlgProc = view_general_proc;
	hpsp[0] = CreatePropertySheetPage(&psp);

	// f[^
	psp.pszTemplate = MAKEINTRESOURCE(IDD_DIALOG_VIEW_DATA);
	psp.pfnDlgProc = view_data_proc;
	hpsp[1] = CreatePropertySheetPage(&psp);

	ZeroMemory(&psh, sizeof(PROPSHEETHEADER));
	psh.dwSize = sizeof_PROPSHEETHEADER;
	psh.dwFlags = PSH_NOAPPLYNOW;
	psh.hInstance = hInst;
	psh.hwndParent = hWnd;
	psh.pszCaption = message_get_res(IDS_STRING_VIEW_TITLE);
	psh.nPages = sizeof(hpsp) / sizeof(HPROPSHEETPAGE);
	psh.phpage = hpsp;

	psh.nStartPage = 0;

	PropertySheet(&psh);
	return 0;
}
/* End of source */
